@@ -1,18 +1,59 @@
 /*
- * ntpd.h - Prototypes for ntpd.
+ * ntpd.h - Prototypes and external variables for ntpd.
+ *
+ * Note the first half is primarily function prototypes, type
+ * declarations, and preprocessor macros, with variables declared
+ * primarily in the second half.
+ *
+ * Each half is further divided into sections for each source file.
  */
 
-#include "ntp_syslog.h"
-#include "ntp_fp.h"
 #include "ntp.h"
+#include "ntp_stdlib.h"
+#include "ntp_syslog.h"
 #include "ntp_debug.h"
+#include "ntp_syslog.h"
 #include "ntp_select.h"
 #include "ntp_malloc.h"
 #include "ntp_refclock.h"
+#include "ntp_intres.h"
 #include "recvbuff.h"
 
-/* ntp_config.c */
+/*
+ * First half: ntpd types, functions, macros
+ * -----------------------------------------
+ */
 
+/*
+ * macro for debugging output - cut down on #ifdef pollution.
+ *
+ * DPRINTF() is for use by ntpd only, and compiles away to nothing
+ * without DEBUG (configure --disable-debugging).
+ *
+ * TRACE() is similar for libntp and utilities, which retain full
+ * debug capability even when compiled without DEBUG.
+ *
+ * The calling convention is not attractive:
+ *     DPRINTF(debuglevel, (fmt, ...));
+ *     DPRINTF(2, ("shows #ifdef DEBUG and if debug >= %d\n", 2));
+ */
+#ifdef DEBUG
+# define DPRINTF(lvl, arg)				\
+	do { 						\
+		if (debug >= (lvl))			\
+			mprintf arg;			\
+	} while (0)
+#else
+# define DPRINTF(lvl, arg)	do {} while (0)
+#endif
+
+
+/* nt_clockstuff.c */
+#ifdef SYS_WINNT
+extern	void	win_time_stepped(void);
+#endif
+
+/* ntp_config.c */
 #define	TAI_1972	10	/* initial TAI offset (s) */
 extern	char	*keysdir;	/* crypto keys and leaptable directory */
 extern	char *	saveconfigdir;	/* ntpq saveconfig output directory */
@@ -24,10 +65,10 @@ extern	u_short ctlpeerstatus	(struct peer *);
 extern	int	ctlsettrap	(sockaddr_u *, struct interface *, int, int);
 extern	u_short ctlsysstatus	(void);
 extern	void	init_control	(void);
-extern	void	init_logging	(char const *, int);
-extern	void	setup_logfile	(void);
 extern	void	process_control (struct recvbuf *, int);
 extern	void	report_event	(int, struct peer *, const char *);
+extern	int	mprintf_event	(int, struct peer *, const char *, ...)
+			NTP_PRINTF(3, 4);
 
 /* ntp_control.c */
 /*
@@ -37,7 +78,7 @@ extern	void	report_event	(int, struct peer *, const char *);
 struct ctl_var {
 	u_short code;
 	u_short flags;
-	char *text;
+	const char *text;
 };
 /*
  * Flag values
@@ -53,18 +94,11 @@ struct ctl_var {
 #define	WO	(CAN_WRITE)
 #define	RW	(CAN_READ|CAN_WRITE)
 
-extern  char *  add_var (struct ctl_var **, u_long, u_short);
-extern  void    free_varlist (struct ctl_var *);
-extern  void    set_var (struct ctl_var **, const char *, u_long, u_short);
-extern  void    set_sys_var (const char *, u_long, u_short);
-
-/* ntp_intres.c */
-extern	void	ntp_res_name	(sockaddr_u, u_short);
-extern	void	ntp_res_recv	(void);
-extern	void	ntp_intres	(void);
-#ifdef SYS_WINNT
-extern	unsigned WINAPI	ntp_intres_thread	(void *);
-#endif
+extern	char *	add_var (struct ctl_var **, u_long, u_short);
+extern	void	free_varlist (struct ctl_var *);
+extern	void	set_var (struct ctl_var **, const char *, u_long, u_short);
+extern	void	set_sys_var (const char *, u_long, u_short);
+extern	const char *	get_ext_sys_var(const char *tag);
 
 /* ntp_io.c */
 typedef struct interface_info {
@@ -74,27 +108,26 @@ typedef struct interface_info {
 
 typedef void	(*interface_receiver_t)	(void *, interface_info_t *);
 
-extern  int	disable_dynamic_updates;
-
 extern	void	interface_enumerate	(interface_receiver_t, void *);
+extern	endpt *	getinterface		(sockaddr_u *, u_int32);
+extern	endpt *	select_peerinterface	(struct peer *, sockaddr_u *,
+					 endpt *);
 extern	endpt *	findinterface		(sockaddr_u *);
 extern	endpt *	findbcastinter		(sockaddr_u *);
 extern	void	enable_broadcast	(endpt *, sockaddr_u *);
 extern	void	enable_multicast_if	(endpt *, sockaddr_u *);
 extern	void	interface_update	(interface_receiver_t, void *);
-
+#ifndef HAVE_IO_COMPLETION_PORT
+extern  void    io_handler              (void);
+#endif
 extern	void	init_io 	(void);
 extern	void	io_open_sockets	(void);
-extern	void	input_handler	(l_fp *);
 extern	void	io_clr_stats	(void);
 extern	void	io_setbclient	(void);
 extern	void	io_unsetbclient	(void);
 extern	void	io_multicast_add(sockaddr_u *);
 extern	void	io_multicast_del(sockaddr_u *);
 extern	void	sendpkt 	(sockaddr_u *, struct interface *, int, struct pkt *, int);
-#ifndef SYS_WINNT
-extern	void	kill_asyncio	(int);
-#endif
 #ifdef DEBUG
 extern	void	collect_timing  (struct recvbuf *, const char *, int, l_fp *);
 #endif
@@ -102,48 +135,63 @@ extern	void	collect_timing  (struct recvbuf *, const char *, int, l_fp *);
 extern	void	wait_for_signal		(void);
 extern	void	unblock_io_and_alarm	(void);
 extern	void	block_io_and_alarm	(void);
-#define UNBLOCK_IO_AND_ALARM()		unblock_io_and_alarm()
-#define BLOCK_IO_AND_ALARM()		block_io_and_alarm()
+# define	UNBLOCK_IO_AND_ALARM()	unblock_io_and_alarm()
+# define	BLOCK_IO_AND_ALARM()	block_io_and_alarm()
 #else
-#define UNBLOCK_IO_AND_ALARM()
-#define BLOCK_IO_AND_ALARM()
+# define	UNBLOCK_IO_AND_ALARM()	do {} while (0)
+# define	BLOCK_IO_AND_ALARM()	do {} while (0)
 #endif
+#define		latoa(pif)	localaddrtoa(pif)
+extern const char * localaddrtoa(endpt *);
 
 /* ntp_loopfilter.c */
 extern	void	init_loopfilter(void);
 extern	int 	local_clock(struct peer *, double);
 extern	void	adj_host_clock(void);
 extern	void	loop_config(int, double);
+extern	void	select_loop(int);
 extern	void	huffpuff(void);
 extern	u_long	sys_clocktime;
 extern	u_int	sys_tai;
+extern 	int	freq_cnt;
 
 /* ntp_monitor.c */
+#define MON_HASH_SIZE		((size_t)1U << mon_hash_bits)
+#define MON_HASH_MASK		(MON_HASH_SIZE - 1)
+#define	MON_HASH(addr)		(sock_hash(addr) & MON_HASH_MASK)
 extern	void	init_mon	(void);
 extern	void	mon_start	(int);
 extern	void	mon_stop	(int);
-extern	int	ntp_monitor	(struct recvbuf *, int);
-extern  void    ntp_monclearinterface (struct interface *interface);
+extern	u_short	ntp_monitor	(struct recvbuf *, u_short);
+extern	void	mon_clearinterface(endpt *interface);
 
 /* ntp_peer.c */
 extern	void	init_peer	(void);
-extern	struct peer *findexistingpeer (sockaddr_u *, struct peer *, int);
+extern	struct peer *findexistingpeer(sockaddr_u *, const char *,
+				      struct peer *, int, u_char);
 extern	struct peer *findpeer	(struct recvbuf *, int, int *);
-extern	struct peer *findpeerbyassoc (u_int);
-extern  void	set_peerdstadr	(struct peer *peer, struct interface *interface);
-extern	struct peer *newpeer	(sockaddr_u *, struct interface *, int, int, int, int, u_int, u_char, int, keyid_t);
+extern	struct peer *findpeerbyassoc(associd_t);
+extern  void	set_peerdstadr	(struct peer *, endpt *);
+extern	struct peer *newpeer	(sockaddr_u *, const char *,
+				 endpt *, u_char, u_char,
+				 u_char, u_char, u_int, u_char, u_int32,
+				 keyid_t, const char *);
 extern	void	peer_all_reset	(void);
 extern	void	peer_clr_stats	(void);
-extern	struct peer *peer_config (sockaddr_u *, struct interface *, int, int, int, int, u_int, int, keyid_t, u_char *);
+extern	struct peer *peer_config(sockaddr_u *, const char *,
+				 endpt *, u_char, u_char,
+				 u_char, u_char, u_int, u_int32,
+				 keyid_t, const char *);
 extern	void	peer_reset	(struct peer *);
-extern	void	refresh_all_peerinterfaces (void);
+extern	void	refresh_all_peerinterfaces(void);
 extern	void	unpeer		(struct peer *);
 extern	void	clear_all	(void);
 extern	int	score_all	(struct peer *);
-extern	struct	peer *findmanycastpeer	(struct recvbuf *);
+extern	struct peer *findmanycastpeer(struct recvbuf *);
+extern	void	peer_cleanup	(void);
 
 /* ntp_crypto.c */
-#ifdef OPENSSL
+#ifdef AUTOKEY
 extern	int	crypto_recv	(struct peer *, struct recvbuf *);
 extern	int	crypto_xmit	(struct peer *, struct pkt *,
 				    struct recvbuf *, int,
@@ -153,6 +201,7 @@ extern	keyid_t	session_key	(sockaddr_u *, sockaddr_u *, keyid_t,
 extern	int	make_keylist	(struct peer *, struct interface *);
 extern	void	key_expire	(struct peer *);
 extern	void	crypto_update	(void);
+extern	void	crypto_update_taichange(void);
 extern	void	crypto_config	(int, char *);
 extern	void	crypto_setup	(void);
 extern	u_int	crypto_ident	(struct peer *);
@@ -166,40 +215,34 @@ extern	EVP_PKEY *gqpar_pkey;
 extern	char	*mvpar_file;
 extern	EVP_PKEY *mvpar_pkey;
 extern struct value tai_leap;
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 /* ntp_proto.c */
 extern	void	transmit	(struct peer *);
 extern	void	receive 	(struct recvbuf *);
-extern	void	peer_clear	(struct peer *, char *);
+extern	void	peer_clear	(struct peer *, const char *);
 extern	void 	process_packet	(struct peer *, struct pkt *, u_int);
 extern	void	clock_select	(void);
+extern	void	set_sys_leap	(u_char);
 
-extern	int	leap_tai;	/* TAI at next leap */
-extern	u_long	leap_sec;	/* next scheduled leap from file */
-extern	u_long	leap_peers;	/* next scheduled leap from peers */
-extern	u_long	leapsec;	/* seconds to next leap */
-extern	u_long	leap_expire;	/* leap information expiration */
+extern	u_long	leapsec;	/* seconds to next leap (proximity class) */
+extern  int     leapdif;        /* TAI difference step at next leap second*/
 extern	int	sys_orphan;
 extern	double	sys_mindisp;
 extern	double	sys_maxdist;
 
-/*
- * there seems to be a bug in the IRIX 4 compiler which prevents
- * u_char from beeing used in prototyped functions.
- * This is also true AIX compiler.
- * So give up and define it to be int. WLJ
- */
-extern	void	poll_update (struct peer *, int);
+extern	char	*sys_ident;	/* identity scheme */
+extern	void	poll_update	(struct peer *, u_char);
 
 extern	void	clear		(struct peer *);
 extern	void	clock_filter	(struct peer *, double, double, double);
 extern	void	init_proto	(void);
+extern	void	set_sys_tick_precision(double);
 extern	void	proto_config	(int, u_long, double, sockaddr_u *);
 extern	void	proto_clr_stats (void);
 
-#ifdef	REFCLOCK
 /* ntp_refclock.c */
+#ifdef	REFCLOCK
 extern	int	refclock_newpeer (struct peer *);
 extern	void	refclock_unpeer (struct peer *);
 extern	void	refclock_receive (struct peer *);
@@ -210,26 +253,30 @@ extern	void	init_refclock	(void);
 /* ntp_request.c */
 extern	void	init_request	(void);
 extern	void	process_private (struct recvbuf *, int);
+extern	void	reset_auth_stats(void);
 
 /* ntp_restrict.c */
 extern	void	init_restrict	(void);
 extern	u_short	restrictions	(sockaddr_u *);
-extern	void	hack_restrict	(int, sockaddr_u *, sockaddr_u *, u_short, u_short);
+extern	void	hack_restrict	(int, sockaddr_u *, sockaddr_u *,
+				 u_short, u_short, u_long);
+extern	void	restrict_source	(sockaddr_u *, int, u_long);
 
 /* ntp_timer.c */
 extern	void	init_timer	(void);
 extern	void	reinit_timer	(void);
 extern	void	timer		(void);
 extern	void	timer_clr_stats (void);
-extern  void    timer_interfacetimeout (u_long);
-extern  volatile int interface_interval;
-#ifdef OPENSSL
+extern	void	timer_interfacetimeout (u_long);
+extern	volatile int interface_interval;
+extern	u_long	orphwait;		/* orphan wait time */
+#ifdef AUTOKEY
 extern	char	*sys_hostname;	/* host name */
 extern	char	*sys_groupname;	/* group name */
 extern	char	*group_name;	/* group name */
-extern u_long	sys_revoke;	/* keys revoke timeout */
-extern u_long	sys_automax;	/* session key timeout */
-#endif /* OPENSSL */
+extern	u_long	sys_revoke;	/* keys revoke timeout */
+extern	u_long	sys_automax;	/* session key timeout */
+#endif	/* AUTOKEY */
 
 /* ntp_util.c */
 extern	void	init_util	(void);
@@ -239,22 +286,39 @@ extern	void	record_peer_stats (sockaddr_u *, int, double, double, double, double
 extern	void	record_proto_stats (char *);
 extern	void	record_loop_stats (double, double, double, double, int);
 extern	void	record_clock_stats (sockaddr_u *, const char *);
-extern	void	record_raw_stats (sockaddr_u *, sockaddr_u *, l_fp *, l_fp *, l_fp *, l_fp *);
-extern	u_long	leap_month(u_long);
+extern	int	mprintf_clock_stats(sockaddr_u *, const char *, ...)
+			NTP_PRINTF(2, 3);
+extern	void	record_raw_stats (sockaddr_u *srcadr, sockaddr_u *dstadr, l_fp *t1, l_fp *t2, l_fp *t3, l_fp *t4, int leap, int version, int mode, int stratum, int ppoll, int precision, double root_delay, double root_dispersion, u_int32 refid);
+extern	void	check_leap_file	(int is_daily_check, u_int32 ntptime, const time_t * systime);
 extern	void	record_crypto_stats (sockaddr_u *, const char *);
 #ifdef DEBUG
 extern	void	record_timing_stats (const char *);
 #endif
-extern  u_short	sock_hash (sockaddr_u *);
 extern	char *	fstostr(time_t);	/* NTP timescale seconds */
-extern	double	old_drift;
-extern	int	drift_file_sw;
-extern	double	wander_threshold;
-extern	double	wander_resid;
+
+/* ntpd.c */
+extern	void	parse_cmdline_opts(int *, char ***);
+/*
+ * Signals we catch for debugging.
+ */
+#define MOREDEBUGSIG	SIGUSR1
+#define LESSDEBUGSIG	SIGUSR2
+/*
+ * Signals which terminate us gracefully.
+ */
+#ifndef SYS_WINNT
+# define SIGDIE1	SIGHUP
+# define SIGDIE2	SIGINT
+# define SIGDIE3	SIGQUIT
+# define SIGDIE4	SIGTERM
+#endif /* SYS_WINNT */
+
 
 /*
- * Variable declarations for ntpd.
+ * Last half: ntpd variables
+ * -------------------------
  */
+
 /* ntp_config.c */
 extern char const *	progname;
 extern char	*sys_phone[];		/* ACTS phone numbers */
@@ -263,7 +327,7 @@ extern int	config_priority_override;
 extern int	config_priority;
 #endif
 extern char *ntp_signd_socket;
-extern struct config_tree *cfg_tree_history;
+extern struct config_tree_tag *cfg_tree_history;
 
 #ifdef BC_LIST_FRAMEWORK_NOT_YET_USED
 /*
@@ -300,17 +364,6 @@ extern u_long	numctldatatooshort;	/* data too short for count */
 extern u_long	numctlbadop; 		/* bad op code found in packet */
 extern u_long	numasyncmsgs;		/* number of async messages we've sent */
 
-/* ntp_intres.c */
-extern keyid_t	req_keyid;		/* request keyid */
-extern int	req_keytype;		/* OpenSSL NID such as NID_md5 */
-extern size_t	req_hashlen;		/* digest size for req_keytype */
-extern char *	req_file;		/* name of the file with configuration info */
-#ifdef SYS_WINNT
-extern HANDLE ResolverEventHandle;
-#else
-extern int resolver_pipe_fd[2];  /* used to let the resolver process alert the parent process */
-#endif /* SYS_WINNT */
-
 /*
  * Other statistics of possible interest
  */
@@ -324,23 +377,19 @@ extern volatile u_long handler_calls;	/* number of calls to interrupt handler */
 extern volatile u_long handler_pkts;	/* number of pkts received by handler */
 extern u_long	io_timereset;		/* time counters were reset */
 
-/*
- * Interface stuff
- */
+/* ntp_io.c */
+extern  int	disable_dynamic_updates;
+extern u_int	sys_ifnum;		/* next .ifnum to assign */
 extern endpt *	any_interface;		/* IPv4 wildcard */
 extern endpt *	any6_interface;		/* IPv6 wildcard */
 extern endpt *	loopback_interface;	/* IPv4 loopback for refclocks */
-
-/*
- * File descriptor masks etc. for call to select
- */
-extern fd_set	activefds;
-extern int	maxactivefd;
+extern endpt *	ep_list;		/* linked list */
 
 /* ntp_loopfilter.c */
 extern double	drift_comp;		/* clock frequency (s/s) */
 extern double	clock_stability;	/* clock stability (s/s) */
-extern double	clock_max;		/* max offset before step (s) */
+extern double	clock_max_back;		/* max backward offset before step (s) */
+extern double	clock_max_fwd;		/* max forward offset before step (s) */
 extern double	clock_panic;		/* max offset before panic (s) */
 extern double	clock_phi;		/* dispersion rate (s/s) */
 extern double	clock_minstep;		/* step timeout (s) */
@@ -355,11 +404,13 @@ extern int	pll_status;		/* status bits for kernel pll */
 extern int	ntp_enable;		/* clock discipline enabled */
 extern int	pll_control;		/* kernel support available */
 extern int	kern_enable;		/* kernel support enabled */
-extern int	pps_enable;		/* kernel PPS discipline enabled */
+extern int	hardpps_enable;		/* kernel PPS discipline enabled */
 extern int	ext_enable;		/* external clock enabled */
 extern int	cal_enable;		/* refclock calibrate enable */
-extern int	allow_panic;		/* allow panic correction */
-extern int	mode_ntpdate;		/* exit on first clock set */
+extern int	allow_panic;		/* allow panic correction (-g) */
+extern int	enable_panic_check;	/* Can we check allow_panic's state? */
+extern int	force_step_once;	/* always step time once at startup (-G) */
+extern int	mode_ntpdate;		/* exit on first clock set (-q) */
 extern int	peer_ntpdate;		/* count of ntpdate peers */
 
 /*
@@ -376,16 +427,28 @@ extern double	sys_offset;		/* system offset (s) */
 extern double	sys_jitter;		/* system jitter (s) */
 
 /* ntp_monitor.c */
-extern struct mon_data mon_mru_list;
-extern struct mon_data mon_fifo_list;
-extern int	mon_enabled;
+extern u_char	mon_hash_bits;		/* log2 size of hash table */
+extern mon_entry ** mon_hash;		/* MRU hash table */
+extern mon_entry mon_mru_list;		/* mru listhead */
+extern u_int	mon_enabled;		/* MON_OFF (0) or other MON_* */
+extern u_int	mru_alloc;		/* mru list + free list count */
+extern u_int	mru_entries;		/* mru list count */
+extern u_int	mru_peakentries;	/* highest mru_entries */
+extern u_int	mru_initalloc;		/* entries to preallocate */
+extern u_int	mru_incalloc;		/* allocation batch factor */
+extern u_int	mru_mindepth;		/* preempt above this */
+extern int	mru_maxage;		/* for entries older than */
+extern u_int	mru_maxdepth; 		/* MRU size hard limit */
+extern int	mon_age;		/* preemption limit */
 
 /* ntp_peer.c */
-extern struct peer *peer_hash[];	/* peer hash table */
-extern int	peer_hash_count[];	/* count of peers in each bucket */
-extern struct peer *assoc_hash[];	/* association ID hash table */
-extern int	assoc_hash_count[];
-extern int	peer_free_count;
+extern struct peer *peer_hash[NTP_HASH_SIZE];	/* peer hash table */
+extern int	peer_hash_count[NTP_HASH_SIZE];	/* count of in each bucket */
+extern struct peer *assoc_hash[NTP_HASH_SIZE];	/* association ID hash table */
+extern int	assoc_hash_count[NTP_HASH_SIZE];/* count of in each bucket */
+extern struct peer *peer_list;		/* peer structures list */
+extern int	peer_count;		/* count in peer_list */
+extern int	peer_free_count;	/* count in peer_free */
 
 /*
  * Miscellaneous statistic counters which may be queried.
@@ -398,6 +461,7 @@ extern u_long	peer_demobilizations;	/* number of structs freed to free list */
 extern int	total_peer_structs;	/* number of peer structs in circulation */
 extern int	peer_associations;	/* mobilized associations */
 extern int	peer_preempt;		/* preemptable associations */
+
 /* ntp_proto.c */
 /*
  * System variables are declared here.	See Section 3.2 of the
@@ -419,6 +483,8 @@ extern int	sys_bclient;		/* we set our time to broadcasts */
 extern double	sys_bdelay; 		/* broadcast client default delay */
 extern int	sys_authenticate;	/* requre authentication for config */
 extern l_fp	sys_authdelay;		/* authentication delay */
+extern u_char	sys_bcpollbstep;	/* broadcast poll backstep gate */
+extern u_long 	sys_epoch;		/* last clock update time */
 extern keyid_t	sys_private;		/* private value for session seed */
 extern int	sys_manycastserver;	/* respond to manycast client pkts */
 extern int	sys_minclock;		/* minimum survivors */
@@ -444,45 +510,65 @@ extern u_long	sys_declined;		/* declined */
 extern u_long	sys_limitrejected;	/* rate exceeded */
 extern u_long	sys_kodsent;		/* KoD sent */
 
-/* ntp_refclock.c */
-#ifdef REFCLOCK
-#ifdef PPS
-extern int	fdpps;			/* pps file descriptor */
-#endif /* PPS */
-#endif
-
 /* ntp_request.c */
 extern keyid_t	info_auth_keyid;	/* keyid used to authenticate requests */
+extern u_long	auth_timereset;
 
 /* ntp_restrict.c */
 extern restrict_u *	restrictlist4;	/* IPv4 restriction list */
 extern restrict_u *	restrictlist6;	/* IPv6 restriction list */
-extern int	ntp_minpkt;
-extern int	ntp_minpoll;
-extern int	mon_age;		/* monitor preempt age */
+extern int		ntp_minpkt;
+extern u_char		ntp_minpoll;
+
+/* ntp_scanner.c */
+extern u_int32		conf_file_sum;	/* Simple sum of characters */
+
+/* ntp_signd.c */
+#ifdef HAVE_NTP_SIGND
+extern void send_via_ntp_signd(struct recvbuf *, int, keyid_t, int,
+			       struct pkt *);
+#endif
 
 /* ntp_timer.c */
 extern volatile int alarm_flag;		/* alarm flag */
 extern volatile u_long alarm_overflow;
-extern u_long	current_time;		/* current time (s) */
+extern u_long	current_time;		/* seconds since startup */
 extern u_long	timer_timereset;
 extern u_long	timer_overflows;
 extern u_long	timer_xmtcalls;
+extern int	leap_sec_in_progress;
+#ifdef LEAP_SMEAR
+extern struct leap_smear_info leap_smear;
+extern int	leap_smear_intv;
+#endif
+#ifdef SYS_WINNT
+HANDLE WaitableTimerHandle;
+#endif
 
 /* ntp_util.c */
-extern int	stats_control;		/* write stats to fileset? */
-extern int	stats_write_period;	/* # of seconds between writes. */
-extern double	stats_write_tolerance;
+extern	char	statsdir[MAXFILENAME];
+extern	int	stats_control;		/* write stats to fileset? */
+extern	int	stats_write_period;	/* # of seconds between writes. */
+extern	double	stats_write_tolerance;
+extern	double	wander_threshold;
 
 /* ntpd.c */
-extern volatile int debug;		/* debugging flag */
-extern int	nofork;			/* no-fork flag */
-extern int 	initializing;		/* initializing flag */
+extern	int	nofork;		/* no-fork flag */
+extern	int	initializing;	/* initializing flag */
 #ifdef HAVE_DROPROOT
-extern int droproot;			/* flag: try to drop root privileges after startup */
-extern char *user;			/* user to switch to */
-extern char *group;			/* group to switch to */
-extern const char *chrootdir;		/* directory to chroot to */
+extern	int	droproot;	/* flag: try to drop root privileges after startup */
+extern	int	root_dropped;	/* root has been dropped */
+extern char *user;		/* user to switch to */
+extern char *group;		/* group to switch to */
+extern const char *chrootdir;	/* directory to chroot() to */
+#endif
+#ifdef HAVE_WORKING_FORK
+extern	int	waitsync_fd_to_close;	/* -w/--wait-sync */
+#endif
+
+/* ntservice.c */
+#ifdef SYS_WINNT
+extern int accept_wildcard_if_for_winnt;
 #endif
 
 /* refclock_conf.c */
@@ -492,14 +578,3 @@ extern struct refclock * const refclock_conf[];
 extern u_char	num_refclock_conf;
 #endif
 
-/* ntp_signd.c */
-#ifdef HAVE_NTP_SIGND
-extern void 
-send_via_ntp_signd(
-	struct recvbuf *rbufp,	/* receive packet pointer */
-	int	xmode,
-	keyid_t	xkeyid, 
-	int flags,
-	struct pkt  *xpkt
-	);
-#endif
