@@ -6,6 +6,8 @@
 #include <config.h>
 #endif
 
+#include "ntp_types.h"
+
 #if defined(REFCLOCK) && defined(CLOCK_ARCRON_MSF)
 
 static const char arc_version[] = { "V1.3 2003/02/21" };
@@ -402,7 +404,7 @@ Also note h<cr> command which starts a resync to MSF signal.
 				       (BITSPERCHAR * BITTIME) ) )
 
      /* Allow for UART to accept char half-way through final stop bit. */
-#define INITIALOFFSET (u_int32)(-BITTIME/2)
+#define INITIALOFFSET ((u_int32)(-BITTIME/2))
 
      /*
     charoffsets[x] is the time after the start of the second that byte
@@ -560,7 +562,7 @@ struct  refclock refclock_arc = {
 /* Queue us up for the next tick. */
 #define ENQUEUE(up) \
 	do { \
-	     peer->nextaction = current_time + QUEUETICK; \
+	     peer->procptr->nextaction = current_time + QUEUETICK; \
 	} while(0)
 
 /* Placeholder event handler---does nothing safely---soaks up loose tick. */
@@ -591,7 +593,7 @@ arc_event_handler(
 	)
 {
 	struct refclockproc *pp = peer->procptr;
-	register struct arcunit *up = (struct arcunit *)pp->unitptr;
+	register struct arcunit *up = pp->unitptr;
 	int i;
 	char c;
 #ifdef DEBUG
@@ -627,50 +629,52 @@ arc_start(
 {
 	register struct arcunit *up;
 	struct refclockproc *pp;
+	int temp_fd;
 	int fd;
 	char device[20];
 #ifdef HAVE_TERMIOS
 	struct termios arg;
 #endif
 
-	msyslog(LOG_NOTICE, "ARCRON: %s: opening unit %d", arc_version, unit);
-#ifdef DEBUG
-	if(debug) {
-		printf("arc: %s: attempt to open unit %d.\n", arc_version, unit);
-	}
-#endif
-
-	/* Prevent a ridiculous device number causing overflow of device[]. */
-	if((unit < 0) || (unit > 255)) { return(0); }
+	msyslog(LOG_NOTICE, "MSF_ARCRON %s: opening unit %d",
+		arc_version, unit);
+	DPRINTF(1, ("arc: %s: attempt to open unit %d.\n", arc_version,
+		unit));
 
 	/*
 	 * Open serial port. Use CLK line discipline, if available.
 	 */
 	snprintf(device, sizeof(device), DEVICE, unit);
-	if (!(fd = refclock_open(device, SPEED, LDISC_CLK)))
-		return(0);
-#ifdef DEBUG
-	if(debug) { printf("arc: unit %d using open().\n", unit); }
-#endif
+	temp_fd = refclock_open(device, SPEED, LDISC_CLK);
+	if (temp_fd <= 0)
+		return 0;
+	DPRINTF(1, ("arc: unit %d using tty_open().\n", unit));
 	fd = tty_open(device, OPEN_FLAGS, 0777);
-	if(fd < 0) {
-#ifdef DEBUG
-		if(debug) { printf("arc: failed [tty_open()] to open %s.\n", device); }
-#endif
-		return(0);
+	if (fd < 0) {
+		msyslog(LOG_ERR, "MSF_ARCRON(%d): failed second open(%s, 0777): %m.",
+			unit, device);
+		close(temp_fd);
+		return 0;
 	}
+	close(temp_fd);
+	temp_fd = -1;		/* not used after this, at *this* time. */
 
 #ifndef SYS_WINNT
-	fcntl(fd, F_SETFL, 0); /* clear the descriptor flags */
-#endif
-#ifdef DEBUG
-	if(debug)
-	{ printf("arc: opened RS232 port with file descriptor %d.\n", fd); }
+	if (-1 == fcntl(fd, F_SETFL, 0)) /* clear the descriptor flags */
+		msyslog(LOG_ERR, "MSF_ARCRON(%d): fcntl(F_SETFL, 0): %m.",
+			unit);
+
 #endif
+	DPRINTF(1, ("arc: opened RS232 port with file descriptor %d.\n", fd));
 
 #ifdef HAVE_TERMIOS
 
-	tcgetattr(fd, &arg);
+	if (tcgetattr(fd, &arg) < 0) {
+		msyslog(LOG_ERR, "MSF_ARCRON(%d): tcgetattr(%s): %m.",
+			unit, device);
+		close(fd);
+		return 0;
+	}
 
 	arg.c_iflag = IGNBRK | ISTRIP;
 	arg.c_oflag = 0;
@@ -679,23 +683,27 @@ arc_start(
 	arg.c_cc[VMIN] = 1;
 	arg.c_cc[VTIME] = 0;
 
-	tcsetattr(fd, TCSANOW, &arg);
+	if (tcsetattr(fd, TCSANOW, &arg) < 0) {
+		msyslog(LOG_ERR, "MSF_ARCRON(%d): tcsetattr(%s): %m.",
+			unit, device);
+		close(fd);
+		return 0;
+	}
 
 #else
 
-	msyslog(LOG_ERR, "ARCRON: termios not supported in this driver");
+	msyslog(LOG_ERR, "ARCRON: termios required by this driver");
 	(void)close(fd);
 
 	return 0;
 
 #endif
 
-	up = emalloc(sizeof(*up));
 	/* Set structure to all zeros... */
-	memset(up, 0, sizeof(*up));
+	up = emalloc_zero(sizeof(*up));
 	pp = peer->procptr;
 	pp->io.clock_recv = arc_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
@@ -704,7 +712,7 @@ arc_start(
 		free(up); 
 		return(0); 
 	}
-	pp->unitptr = (caddr_t)up;
+	pp->unitptr = up;
 
 	/*
 	 * Initialize miscellaneous variables
@@ -752,7 +760,7 @@ arc_start(
 	up->quality = MIN_CLOCK_QUALITY;/* Don't trust the clock yet. */
 #endif
 
-	peer->action = arc_event_handler;
+	peer->procptr->action = arc_event_handler;
 
 	ENQUEUE(up);
 
@@ -772,10 +780,10 @@ arc_shutdown(
 	register struct arcunit *up;
 	struct refclockproc *pp;
 
-	peer->action = dummy_event_handler;
+	peer->procptr->action = dummy_event_handler;
 
 	pp = peer->procptr;
-	up = (struct arcunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (-1 != pp->io.fd)
 		io_closeclock(&pp->io);
 	if (NULL != up)
@@ -835,7 +843,7 @@ send_slow(
 static int
 get2(char *p, int *val)
 {
-  if (!isdigit((int)p[0]) || !isdigit((int)p[1])) return 0;
+  if (!isdigit((unsigned char)p[0]) || !isdigit((unsigned char)p[1])) return 0;
   *val = (p[0] - '0') * 10 + p[1] - '0';
   return 1;
 }
@@ -843,7 +851,7 @@ get2(char *p, int *val)
 static int
 get1(char *p, int *val)
 {
-  if (!isdigit((int)p[0])) return 0;
+  if (!isdigit((unsigned char)p[0])) return 0;
   *val = p[0] - '0';
   return 1;
 }
@@ -875,9 +883,9 @@ arc_receive(
 	/*
 	 * Initialize pointers and read the timecode and timestamp
 	 */
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct arcunit *)pp->unitptr;
+	up = pp->unitptr;
 
 
 	/*
@@ -935,7 +943,7 @@ arc_receive(
 #ifdef DEBUG
 		if(debug) { /* Show \r as `R', other non-printing char as `?'. */
 			printf("arc: stamp -->%c<-- (%d chars rcvd)\n",
-			       ((c == '\r') ? 'R' : (isgraph((int)c) ? c : '?')),
+			       ((c == '\r') ? 'R' : (isgraph((unsigned char)c) ? c : '?')),
 			       rbufp->recv_length);
 		}
 #endif
@@ -1000,7 +1008,7 @@ arc_receive(
 					diff = up->lastrec;
 					L_SUB(&diff, &timestamp);
 					printf("arc: adjusted timestamp by -%sms.\n",
-					       mfptoms(diff.l_i, diff.l_f, 3));
+					       mfptoms(diff.l_ui, diff.l_uf, 3));
 				}
 #endif
 			}
@@ -1454,7 +1462,7 @@ request_time(
 	)
 {
 	struct refclockproc *pp = peer->procptr;
-	register struct arcunit *up = (struct arcunit *)pp->unitptr;
+	register struct arcunit *up = pp->unitptr;
 #ifdef DEBUG
 	if(debug) { printf("arc: unit %d: requesting time.\n", unit); }
 #endif
@@ -1485,7 +1493,7 @@ arc_poll(
 	int resync_needed;              /* Should we start a resync? */
 
 	pp = peer->procptr;
-	up = (struct arcunit *)pp->unitptr;
+	up = pp->unitptr;
 #if 0
 	pp->lencode = 0;
 	memset(pp->a_lastcode, 0, sizeof(pp->a_lastcode));
@@ -1575,5 +1583,5 @@ arc_poll(
 }
 
 #else
-int refclock_arc_bs;
+NONEMPTY_TRANSLATION_UNIT
 #endif
