@@ -332,21 +332,19 @@ irig_start(
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	up = emalloc(sizeof(*up));
-	memset(up, 0, sizeof(*up));
+	up = emalloc_zero(sizeof(*up));
 	pp = peer->procptr;
-	pp->unitptr = (caddr_t)up;
 	pp->io.clock_recv = irig_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
 		close(fd);
 		pp->io.fd = -1;
 		free(up);
-		pp->unitptr = NULL;
 		return (0);
 	}
+	pp->unitptr = up;
 
 	/*
 	 * Initialize miscellaneous variables
@@ -390,7 +388,7 @@ irig_shutdown(
 	struct irigunit *up;
 
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (-1 != pp->io.fd)
 		io_closeclock(&pp->io);
 	if (NULL != up)
@@ -421,9 +419,9 @@ irig_receive(
 	int	bufcnt;		/* buffer counter */
 	l_fp	ltemp;		/* l_fp temp */
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Main loop - read until there ain't no more. Note codec
@@ -459,8 +457,8 @@ irig_receive(
 		sample = fabs(sample);
 		if (sample > up->signal)
 			up->signal = sample;
-			up->signal += (sample - up->signal) /
-			    1000;
+		up->signal += (sample - up->signal) /
+		    1000;
 
 		/*
 		 * Once each second, determine the IRIG format and gain.
@@ -519,7 +517,7 @@ irig_rf(
 	double	irig_b, irig_e;	/* irig filter outputs */
 
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * IRIG-B filter. Matlab 4th-order IIR elliptic, 800-1200 Hz
@@ -601,7 +599,7 @@ irig_base(
 	int	carphase;	/* carrier phase */
 
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Synchronous baud integrator. Corresponding samples of current
@@ -754,7 +752,7 @@ irig_baud(
 	l_fp	ltemp;
 
         pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * The PLL time constant starts out small, in order to
@@ -764,7 +762,7 @@ irig_baud(
 	 * persist for lots of samples.
 	 */
 	up->exing = -up->yxing;
-	if (fabs(up->envxing - up->envphase) <= 1) {
+	if (abs(up->envxing - up->envphase) <= 1) {
 		up->tcount++;
 		if (up->tcount > 20 * up->tc) {
 			up->tc++;
@@ -848,8 +846,9 @@ irig_decode(
 	char	spare[2 + 1];	/* mulligan digits */
 	int	temp;
 
+	syncdig = 0;
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Assemble frame bits.
@@ -981,10 +980,8 @@ irig_poll(
 	)
 {
 	struct refclockproc *pp;
-	struct irigunit *up;
 
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
 
 	if (pp->coderecv == pp->codeproc) {
 		refclock_report(peer, CEVNT_TIMEOUT);
@@ -1021,7 +1018,7 @@ irig_gain(
 	struct irigunit *up;
 
 	pp = peer->procptr;
-	up = (struct irigunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Apparently, the codec uses only the high order bits of the
