@@ -65,6 +65,10 @@
  *
  * 30/08/09: Added support for Trimble Acutime Gold Receiver.
  *	     Fernando P. Hauscarriaga (fernandoph@iar.unlp.edu.ar)
+ *
+ * 21/04/18: Added support for Resolution devices.
+ *
+ * 03/09/19: Added support for ACE III & Copernicus II.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -80,10 +84,6 @@ extern int async_write(int, const void *, unsigned int);
 #endif
 
 #include "refclock_palisade.h"
-/* Table to get from month to day of the year */
-const int days_of_year [12] = {
-	0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334
-};
 
 #ifdef DEBUG
 const char * Tracking_Status[15][15] = { 
@@ -107,7 +107,7 @@ struct refclock refclock_palisade = {
 	NOFLAGS			/* not used */
 };
 
-int day_of_year (char *dt);
+static int decode_date(struct refclockproc *pp, const char *cp);
 
 /* Extract the clock type from the mode setting */
 #define CLK_TYPE(x) ((int)(((x)->ttl) & 0x7F))
@@ -118,6 +118,9 @@ int day_of_year (char *dt);
 #define CLK_THUNDERBOLT	2	/* Trimble Thunderbolt GPS Receiver */
 #define CLK_ACUTIME     3	/* Trimble Acutime Gold */
 #define CLK_ACUTIMEB    4	/* Trimble Actutime Gold Port B */
+#define CLK_RESOLUTION  5	/* Trimble Resolution Receivers */
+#define CLK_ACE		6	/* Trimble ACE III */
+#define CLK_COPERNICUS	7	/* Trimble Copernicus II */
 
 int praecis_msg;
 static void praecis_parse(struct recvbuf *rbufp, struct peer *peer);
@@ -126,7 +129,6 @@ static void praecis_parse(struct recvbuf *rbufp, struct peer *peer);
  * They are taken from Markus Prosch
  */
 
-#ifdef PALISADE_SENDCMD_RESURRECTED
 /*
  * sendcmd - Build data packet for sending
  */
@@ -140,7 +142,6 @@ sendcmd (
 	*(buffer->data + 1) = (unsigned char)c;
 	buffer->size = 2;
 }
-#endif	/* PALISADE_SENDCMD_RESURRECTED */
 
 /*
  * sendsupercmd - Build super data packet for sending
@@ -218,7 +219,7 @@ init_thunderbolt (
 	struct packettx tx;
 	
 	tx.size = 0;
-	tx.data = (u_char *) malloc(100);
+	tx.data = (u_char *) emalloc(100);
 
 	/* set UTC time */
 	sendsupercmd (&tx, 0x8E, 0xA2);
@@ -226,7 +227,7 @@ init_thunderbolt (
 	sendetx      (&tx, fd);
 	
 	/* activate packets 0x8F-AB and 0x8F-AC */
-	sendsupercmd (&tx, 0x8F, 0xA5);
+	sendsupercmd (&tx, 0x8E, 0xA5);
 	sendint      (&tx, 0x5);
 	sendetx      (&tx, fd);
 
@@ -246,7 +247,7 @@ init_acutime (
 	struct packettx tx;
 
 	tx.size = 0;
-	tx.data = (u_char *) malloc(100);
+	tx.data = (u_char *) emalloc(100);
 
 	sendsupercmd(&tx, 0x8E, 0xA5);
 	sendbyte(&tx, 0x02);
@@ -258,6 +259,37 @@ init_acutime (
 	free(tx.data);
 }
 
+/*
+ * init_resolution - Prepares Resolution receiver to be used with NTP
+ */
+static void
+init_resolution (
+	int fd
+	)
+{
+	struct packettx tx;
+	
+	tx.size = 0;
+	tx.data = (u_char *) emalloc(100);
+
+	/* set UTC time */
+	sendsupercmd (&tx, 0x8E, 0xA2);
+	sendbyte     (&tx, 0x3);
+	sendetx      (&tx, fd);
+
+	/* squelch PPS output unless locked to at least one satellite */
+	sendsupercmd (&tx, 0x8E, 0x4E);
+	sendbyte     (&tx, 0x3);
+	sendetx      (&tx, fd);
+	
+	/* activate packets 0x8F-AB and 0x8F-AC */
+	sendsupercmd (&tx, 0x8E, 0xA5);
+	sendint      (&tx, 0x5);
+	sendetx      (&tx, fd);
+
+	free(tx.data);
+}
+
 /*
  * palisade_start - open the devices and initialize data for processing
  */
@@ -272,13 +304,15 @@ palisade_start (
 	int fd;
 	char gpsdev[20];
 	struct termios tio;
+	u_int speed;
 
 	snprintf(gpsdev, sizeof(gpsdev), DEVICE, unit);
 
 	/*
 	 * Open serial port. 
 	 */
-	fd = refclock_open(gpsdev, SPEED232, LDISC_RAW);
+	speed = (CLK_TYPE(peer) == CLK_COPERNICUS) ? SPEED232COP : SPEED232;
+	fd = refclock_open(&peer->srcadr, gpsdev, speed, LDISC_RAW);
 	if (fd <= 0) {
 #ifdef DEBUG
 		printf("Palisade(%d) start: open %s failed\n", unit, gpsdev);
@@ -305,9 +339,7 @@ palisade_start (
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	up = (struct palisade_unit *) emalloc(sizeof(struct palisade_unit));
-
-	memset((char *)up, 0, sizeof(struct palisade_unit));
+	up = emalloc_zero(sizeof(*up));
 
 	up->type = CLK_TYPE(peer);
 	switch (up->type) {
@@ -315,20 +347,39 @@ palisade_start (
 		/* Normal mode, do nothing */
 		break;
 	    case CLK_PRAECIS:
-		msyslog(LOG_NOTICE, "Palisade(%d) Praecis mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Praecis mode enabled"
 			,unit);
 		break;
 	    case CLK_THUNDERBOLT:
-		msyslog(LOG_NOTICE, "Palisade(%d) Thunderbolt mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Thunderbolt mode enabled"
 			,unit);
 		tio.c_cflag = (CS8|CLOCAL|CREAD);
 		break;
 	    case CLK_ACUTIME:
-		msyslog(LOG_NOTICE, "Palisade(%d) Acutime Gold mode enabled\n"
+		msyslog(LOG_NOTICE, "Palisade(%d) Acutime Gold mode enabled"
 			,unit);
 		break;
+	    case CLK_RESOLUTION:
+		msyslog(LOG_NOTICE, "Palisade(%d) Resolution mode enabled"
+			,unit);
+		tio.c_cflag = (CS8|CLOCAL|CREAD|PARENB|PARODD);
+		break;
+	    case CLK_ACE:
+		msyslog(LOG_NOTICE, "Palisade(%d) ACE III mode enabled"
+			,unit);
+		tio.c_cflag = (CS8|CLOCAL|CREAD|PARENB|PARODD);
+		break;
+	    case CLK_COPERNICUS:
+		msyslog(LOG_NOTICE, "Palisade(%d) Copernicus II mode enabled"
+			,unit);
+		/* Must use ORing/ANDing to set/clear c_cflag bits otherwise
+		   CBAUD gets set back to 0. This ought to be an issue for
+		   the other modes above but it seems that the baud rate
+		   defaults to 9600 if CBAUD gets set to 0.                 */
+		tio.c_cflag &= ~(PARENB|PARODD);
+		break;
 	    default:
-		msyslog(LOG_NOTICE, "Palisade(%d) mode unknown\n",unit);
+		msyslog(LOG_NOTICE, "Palisade(%d) mode unknown",unit);
 		break;
 	}
 	if (tcsetattr(fd, TCSANOW, &tio) == -1) {
@@ -343,7 +394,7 @@ palisade_start (
 
 	pp = peer->procptr;
 	pp->io.clock_recv = palisade_io;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
@@ -359,7 +410,7 @@ palisade_start (
 	/*
 	 * Initialize miscellaneous variables
 	 */
-	pp->unitptr = (caddr_t)up;
+	pp->unitptr = up;
 	pp->clockdesc = DESCRIPTION;
 
 	peer->precision = PRECISION;
@@ -377,6 +428,8 @@ palisade_start (
 		init_thunderbolt(fd);
 	if (up->type == CLK_ACUTIME)
 		init_acutime(fd);
+	if (up->type == CLK_RESOLUTION)
+		init_resolution(fd);
 
 	return 1;
 }
@@ -394,7 +447,7 @@ palisade_shutdown (
 	struct palisade_unit *up;
 	struct refclockproc *pp;
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 	if (-1 != pp->io.fd)
 		io_closeclock(&pp->io);
 	if (NULL != up)
@@ -402,33 +455,78 @@ palisade_shutdown (
 }
 
 
-
 /* 
- * unpack_date - get day and year from date
+ * unpack helpers
  */
-int
-day_of_year (
-	char * dt
-	)
+
+static inline uint8_t
+get_u8(
+	const char *cp)
 {
-	int day, mon, year;
-
-	mon = dt[1];
-	/* Check month is inside array bounds */
-	if ((mon < 1) || (mon > 12)) 
-		return -1;
-
-	day = dt[0] + days_of_year[mon - 1];
-	year = getint((u_char *) (dt + 2)); 
-
-	if ( !(year % 4) && ((year % 100) || 
-			     (!(year % 100) && !(year%400)))
-	     &&(mon > 2))
-		day ++; /* leap year and March or later */
-
-	return day;
+	return ((const u_char*)cp)[0];
 }
 
+static inline uint16_t
+get_u16(
+	const char *cp)
+{
+	return ((uint16_t)get_u8(cp) << 8) | get_u8(cp + 1);
+}
+
+/*
+ * unpack & fix date (the receiver provides a valid time for 1024 weeks
+ * after 1997-12-14 and therefore folds back in 2017, 2037,...)
+ *
+ * Returns -1 on error, day-of-month + (month * 32) othertwise.
+ */
+int
+decode_date(
+	struct refclockproc *pp,
+	const char          *cp)
+{
+	static int32_t  s_baseday = 0;
+	
+	struct calendar jd;
+	int32_t         rd;
+
+	if (0 == s_baseday) {
+		if (!ntpcal_get_build_date(&jd)) {
+			jd.year     = 2015;
+			jd.month    = 1;
+			jd.monthday = 1;
+		}
+		s_baseday = ntpcal_date_to_rd(&jd);
+	}
+
+	/* get date fields and convert to RDN */
+	jd.monthday = get_u8 (  cp  );
+	jd.month    = get_u8 (cp + 1);
+	jd.year     = get_u16(cp + 2);
+	rd = ntpcal_date_to_rd(&jd);
+
+	/* for the paranoid: do reverse calculation and cross-check */
+	ntpcal_rd_to_date(&jd, rd);
+	if ((jd.monthday != get_u8 (  cp  )) ||
+	    (jd.month    != get_u8 (cp + 1)) ||
+	    (jd.year     != get_u16(cp + 2))  )
+		return - 1;
+	
+	/* calculate cycle shift to base day and calculate re-folded
+	 * date
+	 *
+	 * One could do a proper modulo calculation here, but a counting
+	 * loop is probably faster for the next few rollovers...
+	 */
+	while (rd < s_baseday)
+		rd += 7*1024;
+	ntpcal_rd_to_date(&jd, rd);
+
+	/* fill refclock structure & indicate success */
+	pp->day  = jd.yearday;
+	pp->year = jd.year;	
+	return ((int)jd.month << 5) | jd.monthday;
+}
+    
 
 /* 
  * TSIP_decode - decode the TSIP data packets 
@@ -443,12 +541,16 @@ TSIP_decode (
 	double secs;
 	double secfrac;
 	unsigned short event = 0;
-
+	int mmday;
+	long tow;
+	uint16_t wn;
+	int GPS_UTC_Offset;
+	
 	struct palisade_unit *up;
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Check the time packet, decode its contents. 
@@ -456,7 +558,12 @@ TSIP_decode (
 	 * proper format, declare bad format and exit.
 	 */
 
-	if ((up->type != CLK_THUNDERBOLT) & (up->type != CLK_ACUTIME)){
+	if ((up->type != CLK_THUNDERBOLT) &&
+	    (up->type != CLK_ACUTIME    ) &&
+	    (up->type != CLK_RESOLUTION ) &&
+	    (up->type != CLK_ACE        ) &&
+	    (up->type != CLK_COPERNICUS )   )
+	{
 		if ((up->rpt_buf[0] == (char) 0x41) ||
 		    (up->rpt_buf[0] == (char) 0x46) ||
 		    (up->rpt_buf[0] == (char) 0x54) ||
@@ -473,7 +580,7 @@ TSIP_decode (
 	}
 
 	/*
-	 * We cast both to u_char to as 0x8f uses the sign bit on a char
+	 * We cast both to u_char as 0x8f uses the sign bit on a char
 	 */
 	if ((u_char) up->rpt_buf[0] == (u_char) 0x8f) {
 		/* 
@@ -485,8 +592,6 @@ TSIP_decode (
 			return 0;	   
 	
 		switch (mb(0) & 0xff) {
-			int GPS_UTC_Offset;
-			long tow;
 
 		    case PACKET_8F0B: 
 
@@ -537,16 +642,16 @@ TSIP_decode (
 			pp->minute = secint / 60;
 			secint %= 60;
 			pp->second = secint % 60;
-		
-			if ((pp->day = day_of_year(&mb(11))) < 0) break;
 
-			pp->year = getint((u_char *) &mb(13)); 
+			mmday = decode_date(pp, &mb(11));
+			if (mmday < 0)
+				break;
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d UTC %02d\n",
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d UTC %02d\n",
 				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, 
-				       pp->second, pp->nsec, mb(12), mb(11), pp->year, GPS_UTC_Offset);
+				       pp->second, pp->nsec, (mmday >> 5), (mmday & 31), pp->year, GPS_UTC_Offset);
 #endif
 			/* Only use this packet when no
 			 * 8F-AD's are being received
@@ -586,7 +691,14 @@ TSIP_decode (
 				break;
 			}
 
-			if (up->leap_status & PALISADE_LEAP_PENDING) {
+			mmday = decode_date(pp, &mb(14));
+			if (mmday < 0)
+				break;
+			up->month  = (mmday >> 5);  /* Save for LEAP check */
+
+			if ( (up->leap_status & PALISADE_LEAP_PENDING) &&
+			/* Avoid early announce: https://bugs.ntp.org/2773 */
+				(6 == up->month || 12 == up->month) ) {
 				if (up->leap_status & PALISADE_UTC_TIME)  
 					pp->leap = LEAP_ADDSECOND;
 				else
@@ -611,18 +723,15 @@ TSIP_decode (
 			pp->nsec = (long) (getdbl((u_char *) &mb(3))
 					   * 1000000000);
 
-			if ((pp->day = day_of_year(&mb(14))) < 0) 
-				break;
-			pp->year = getint((u_char *) &mb(16)); 
 			pp->hour = mb(11);
 			pp->minute = mb(12);
 			pp->second = mb(13);
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d UTC %02x %s\n",
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d UTC %02x %s\n",
 				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, 
-				       pp->second, pp->nsec, mb(15), mb(14), pp->year,
+				       pp->second, pp->nsec, (mmday >> 5), (mmday & 31), pp->year,
 				       mb(19), *Tracking_Status[st]);
 #endif
 			return 1;
@@ -647,7 +756,9 @@ TSIP_decode (
 				printf("TSIP_decode: unit %d\n", up->unit);
 			}
 #endif
-			if (getint((u_char *) &mb(10)) & 0x80) 
+			if ( (getint((u_char *) &mb(10)) & 0x80) &&
+			/* Avoid early announce: https://bugs.ntp.org/2773 */
+			    (6 == up->month || 12 == up->month) )
 				pp->leap = LEAP_ADDSECOND;  /* we ASSUME addsecond */
 			else 
 				pp->leap = LEAP_NOWARNING;
@@ -746,17 +857,17 @@ TSIP_decode (
 				printf ("	Time is from GPS\n\n");	
 #endif		
 
-			if ((pp->day = day_of_year(&mb(13))) < 0)
+			mmday = decode_date(pp, &mb(13));
+			if (mmday < 0)
 				break;
 			tow = getlong((u_char *) &mb(1));
 #ifdef DEBUG		
 			if (debug > 1) {
 				printf("pp->day: %d\n", pp->day); 
 				printf("TOW: %ld\n", tow);
-				printf("DAY: %d\n", mb(13));
+				printf("DAY: %d\n", (mmday & 31));
 			}
 #endif
-			pp->year = getint((u_char *) &mb(15));
 			pp->hour = mb(12);
 			pp->minute = mb(11);
 			pp->second = mb(10);
@@ -764,7 +875,9 @@ TSIP_decode (
 
 #ifdef DEBUG
 			if (debug > 1)
-				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%06ld %02d/%02d/%04d ",up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, pp->second, pp->nsec, mb(14), mb(13), pp->year);
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d ",
+				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, pp->second,
+				       pp->nsec, (mmday >> 5), (mmday & 31), pp->year);
 #endif
 			return 1;
 			break;
@@ -789,13 +902,74 @@ TSIP_decode (
 	}
 	else if ((up->rpt_buf[0] == PACKET_41A) & (up->type == CLK_ACUTIME)) {
 #ifdef DEBUG
-		printf("GPS TOW: %ld\n", getlong((u_char *) &mb(0)));
+		printf("GPS TOW: %ld\n", (long)getlong((u_char *) &mb(0)));
 		printf("GPS WN: %d\n", getint((u_char *) &mb(4)));
-		printf("GPS UTC-GPS Offser: %ld\n", getlong((u_char *) &mb(6)));
+		printf("GPS UTC-GPS Offset: %ld\n", (long)getlong((u_char *) &mb(6)));
 #endif
 		return 0;
 	}
 
+	/* GPS time packet for ACE III or Copernicus II receiver */
+	else if ((up->rpt_buf[0] == PACKET_41) &&
+	         ((up->type == CLK_ACE) || (up->type == CLK_COPERNICUS))) {
+#ifdef DEBUG
+		if ((debug > 1) && (up->type == CLK_ACE))
+			printf("TSIP_decode: Packet 0x41 seen in ACE III mode\n");
+		if ((debug > 1) && (up->type == CLK_COPERNICUS))
+			printf("TSIP_decode: Packet 0x41 seen in Copernicus II mode\n");
+#endif
+		if (up->rpt_cnt != LENCODE_41) { /* check length */
+			refclock_report(peer, CEVNT_BADREPLY);
+			up->polled = -1;
+#ifdef DEBUG
+			printf("TSIP_decode: unit %d: bad packet %02x len %d\n", 
+				up->unit, up->rpt_buf[0] & 0xff, up->rpt_cnt);
+#endif
+			return 0;
+		}
+		if (up->polled  <= 0)
+			return 0;
+		tow = (long)getsingle((u_char *) &mb(0));
+		wn = (uint16_t)getint((u_char *) &mb(4));
+		GPS_UTC_Offset = (int)getsingle((u_char *) &mb(6));
+		if (GPS_UTC_Offset == 0){ /* Check UTC Offset */
+#ifdef DEBUG
+			printf("TSIP_decode: UTC Offset Unknown\n");
+#endif
+			refclock_report(peer, CEVNT_BADREPLY);
+			up->polled = -1;
+			return 0;
+		}
+		/* Get date & time from WN & ToW minus offset */
+		{
+			TCivilDate cd;
+			TGpsDatum wd;
+			l_fp ugo; /* UTC-GPS offset, negative number */
+			ugo.Ul_i.Xl_i = (int32_t)-GPS_UTC_Offset;
+			ugo.l_uf = 0;
+			wd = gpscal_from_gpsweek((wn % 1024), (int32_t)tow, ugo);
+			gpscal_to_calendar(&cd, &wd);
+			pp->year = cd.year;
+			pp->day = cd.yearday;
+			pp->hour = cd.hour;
+			pp->minute = cd.minute;
+			pp->second = cd.second;
+			pp->nsec = 0;
+			pp->leap = LEAP_NOWARNING;
+#ifdef DEBUG
+			if (debug > 1)	{
+				printf("GPS TOW: %ld\n", tow);
+				printf("GPS WN: %d\n", wn);
+				printf("GPS UTC-GPS Offset: %d\n", GPS_UTC_Offset);
+				printf("TSIP_decode: unit %d: %02X #%d %02d:%02d:%02d.%09ld %02d/%02d/%04d ",
+				       up->unit, mb(0) & 0xff, event, pp->hour, pp->minute, pp->second,
+				       pp->nsec, cd.month, cd.monthday, pp->year);
+			}
+#endif
+		}
+		return 1;
+	}
+
 	/* Health Status for Acutime Receiver */
 	else if ((up->rpt_buf[0] == PACKET_46) & (up->type == CLK_ACUTIME)) {
 #ifdef DEBUG
@@ -806,7 +980,7 @@ TSIP_decode (
 				printf ("Doing Position Fixes\n");
 				break;
 			    case 0x01:
-				printf ("Do no have GPS time yet\n");
+				printf ("Do not have GPS time yet\n");
 				break;
 			    case 0x03:
 				printf ("PDOP is too high\n");
@@ -835,15 +1009,15 @@ TSIP_decode (
 			up->polled = -1;
 #ifdef DEBUG
 			if (debug > 1) {
-				if (mb(1) && 0x01)
+				if (mb(1) & 0x01)
 					printf ("Signal Processor Error, reset unit.\n");
-				if (mb(1) && 0x02)
+				if (mb(1) & 0x02)
 					printf ("Alignment error, channel or chip 1, reset unit.\n");
-				if (mb(1) && 0x03)
+				if (mb(1) & 0x03)
 					printf ("Alignment error, channel or chip 2, reset unit.\n");
-				if (mb(1) && 0x04)
+				if (mb(1) & 0x04)
 					printf ("Antenna feed line fault (open or short)\n");
-				if (mb(1) && 0x05)
+				if (mb(1) & 0x05)
 					printf ("Excessive reference frequency error, refer to packet 0x2D and packet 0x4D documentation for further information\n");
 			}
 #endif
@@ -851,6 +1025,73 @@ TSIP_decode (
 		return 0;
 		}
 	}
+
+	/* Health Status for Copernicus II Receiver */
+	else if ((up->rpt_buf[0] == PACKET_46) && (up->type == CLK_COPERNICUS)) {
+#ifdef DEBUG
+		if (debug > 1)
+		/* Status Codes */
+			switch (mb(0)) {
+			    case 0x00:
+				printf ("Doing Position Fixes\n");
+				break;
+			    case 0x01:
+				printf ("Do not have GPS time yet\n");
+				break;
+			    case 0x03:
+				printf ("PDOP is too high\n");
+				break;
+			    case 0x04:
+				printf("The Chosen satellite is unusable\n");
+				break;
+			    case 0x08:
+				printf ("No usable satellites\n");
+				break;
+			    case 0x09:
+				printf ("Only 1 usable satellite\n");
+				break;
+			    case 0x0A:
+				printf ("Only 2 usable satellites\n");
+				break;
+			    case 0x0B:
+				printf ("Only 3 usable satellites\n");
+				break;
+			}
+#endif
+		/* Error Codes */
+		if ((mb(1) & 0x3E) != 0) {  /* Don't regard bits 0 and 6 as errors */
+			refclock_report(peer, CEVNT_BADTIME);
+			up->polled = -1;
+#ifdef DEBUG
+			if (debug > 1) {
+				if ((mb(1) & 0x18) == 0x08)
+					printf ("Antenna feed line fault (open)\n");
+				if ((mb(1) & 0x18) == 0x18)
+					printf ("Antenna feed line fault (short)\n");
+			}
+#endif
+		}
+		return 0;
+	}
+
+	/* Other packets output by ACE III & Copernicus II Receivers, dropped silently */
+	else if (((up->rpt_buf[0] == (char) 0x4A) ||
+		  (up->rpt_buf[0] == (char) 0x4B) ||
+		  (up->rpt_buf[0] == (char) 0x56) ||
+		  (up->rpt_buf[0] == (char) 0x5F) ||
+		  (up->rpt_buf[0] == (char) 0x6D) ||
+		  (up->rpt_buf[0] == (char) 0x82) ||
+		  (up->rpt_buf[0] == (char) 0x84)) &&
+		 ((up->type == CLK_ACE) || (up->type == CLK_COPERNICUS))) {
+#ifdef DEBUG
+		if ((debug > 1) && (up->type == CLK_ACE))
+			printf("TSIP_decode: Packet 0x%2x seen in ACE III mode\n", (up->rpt_buf[0] & 0XFF));
+		if ((debug > 1) && (up->type == CLK_COPERNICUS))
+			printf("TSIP_decode: Packet 0x%2x seen in Copernicus II mode\n", (up->rpt_buf[0] & 0XFF));
+#endif
+		return 0;
+	}
+
 	else if (up->rpt_buf[0] == 0x54)
 		return 0;
 
@@ -900,7 +1141,7 @@ palisade_receive (
 	 * Initialize pointers and read the timecode and timestamp.
 	 */
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 		
 	if (! TSIP_decode(peer)) return;
 	
@@ -912,7 +1153,7 @@ palisade_receive (
 #ifdef DEBUG
 	if (debug) 
 		printf(
-			"palisade_receive: unit %d: %4d %03d %02d:%02d:%02d.%06ld\n",
+			"palisade_receive: unit %d: %4d %03d %02d:%02d:%02d.%09ld\n",
 			up->unit, pp->year, pp->day, pp->hour, pp->minute, 
 			pp->second, pp->nsec);
 #endif
@@ -924,7 +1165,7 @@ palisade_receive (
 	 */
 
 	snprintf(pp->a_lastcode, sizeof(pp->a_lastcode),
-		 "%4d %03d %02d:%02d:%02d.%06ld",
+		 "%4d %03d %02d:%02d:%02d.%09ld",
 		 pp->year, pp->day,
 		 pp->hour,pp->minute, pp->second, pp->nsec); 
 	pp->lencode = 24;
@@ -965,7 +1206,7 @@ palisade_poll (
 	struct refclockproc *pp;
 	
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	pp->polls++;
 	if (up->polled > 0) /* last reply never arrived or error */ 
@@ -984,9 +1225,9 @@ palisade_poll (
 		return;  /* using synchronous packet input */
 
 	if(up->type == CLK_PRAECIS) {
-		if(write(peer->procptr->io.fd,"SPSTAT\r\n",8) < 0)
+		if (write(peer->procptr->io.fd,"SPSTAT\r\n",8) < 0) {
 			msyslog(LOG_ERR, "Palisade(%d) write: %m:",unit);
-		else {
+		} else {
 			praecis_msg = 1;
 			return;
 		}
@@ -1008,20 +1249,53 @@ praecis_parse (
 
 	pp = peer->procptr;
 
-	memcpy(buf+p,rbufp->recv_space.X_recv_buffer, rbufp->recv_length);
+	if (p + rbufp->recv_length >= sizeof buf) {
+		struct palisade_unit *up;
+		up = pp->unitptr;
+
+		/*
+		 * We COULD see if there is a \r\n in the incoming
+		 * buffer before it overflows, and then process the
+		 * current line.
+		 *
+		 * Similarly, if we already have a hunk of data that
+		 * we're now flushing, that will cause the line of
+		 * data we're in the process of collecting to be garbage.
+		 *
+		 * Since we now check for this overflow and log when it
+		 * happens, we're now in a better place to easily see
+		 * what's going on and perhaps better choices can be made.
+		 */
+
+		/* Do we need to log the size of the overflow? */
+		msyslog(LOG_ERR, "Palisade(%d) praecis_parse(): input buffer overflow", 
+			up->unit);
+
+		p = 0;
+		praecis_msg = 0;
+
+		refclock_report(peer, CEVNT_BADREPLY);
+
+		return;
+	}
+
+	memcpy(buf+p, rbufp->recv_buffer, rbufp->recv_length);
 	p += rbufp->recv_length;
 
-	if(buf[p-2] == '\r' && buf[p-1] == '\n') {
+	if (   p >= 2
+	    && buf[p-2] == '\r' 
+	    && buf[p-1] == '\n') {
 		buf[p-2] = '\0';
 		record_clock_stats(&peer->srcadr, buf);
 
 		p = 0;
 		praecis_msg = 0;
 
-		if (HW_poll(pp) < 0)
+		if (HW_poll(pp) < 0) {
 			refclock_report(peer, CEVNT_FAULT);
-
+		}
 	}
+	return;
 }
 
 static void
@@ -1038,9 +1312,9 @@ palisade_io (
 
 	char * c, * d;
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct palisade_unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	if(up->type == CLK_PRAECIS) {
 		if(praecis_msg) {
@@ -1140,15 +1414,23 @@ HW_poll (
 {	
 	int x;	/* state before & after RTS set */
 	struct palisade_unit *up;
+	struct packettx tx;
 
-	up = (struct palisade_unit *) pp->unitptr;
+	up = pp->unitptr;
+
+	if (up->type == CLK_ACE) {
+		/* Poll by sending a 0x21 command */
+		tx.size = 0;
+		tx.data = (u_char *) emalloc(100);
+		sendcmd (&tx, 0x21);
+		sendetx (&tx, pp->io.fd);
+		free(tx.data);
+	} else {
 
 	/* read the current status, so we put things back right */
 	if (ioctl(pp->io.fd, TIOCMGET, &x) < 0) {
-#ifdef DEBUG
-		if (debug)
-			printf("Palisade HW_poll: unit %d: GET %s\n", up->unit, strerror(errno));
-#endif
+		DPRINTF(1, ("Palisade HW_poll: unit %d: GET %m\n",
+			up->unit));
 		msyslog(LOG_ERR, "Palisade(%d) HW_poll: ioctl(fd,GET): %m", 
 			up->unit);
 		return -1;
@@ -1158,7 +1440,10 @@ HW_poll (
 
 	/* Edge trigger */
 	if (up->type == CLK_ACUTIME)
-		write (pp->io.fd, "", 1);
+		if (write (pp->io.fd, "", 1) != 1)
+			msyslog(LOG_WARNING,
+				"Palisade(%d) HW_poll: failed to send trigger: %m", 
+				up->unit);
 		
 	if (ioctl(pp->io.fd, TIOCMSET, &x) < 0) { 
 #ifdef DEBUG
@@ -1173,9 +1458,12 @@ HW_poll (
 
 	x &= ~TIOCM_RTS;        /* turn off RTS  */
 	
+	} /* (up->type != CLK_ACE) */
+
 	/* poll timestamp */
 	get_systime(&pp->lastrec);
 
+	if (up->type != CLK_ACE) {
 	if (ioctl(pp->io.fd, TIOCMSET, &x) == -1) {
 #ifdef DEBUG
 		if (debug)
@@ -1186,89 +1474,96 @@ HW_poll (
 			up->unit);
 		return -1;
 	}
+	}
 
 	return 0;
 }
 
-#if 0 /* unused */
 /*
- * this 'casts' a character array into a float
+ * copy/swap a big-endian palisade double into a host double
  */
-float
-getfloat (
-	u_char *bp
-	)
-{
-	float sval;
-#ifdef WORDS_BIGENDIAN 
-	((char *) &sval)[0] = *bp++;
-	((char *) &sval)[1] = *bp++;
-	((char *) &sval)[2] = *bp++;
-	((char *) &sval)[3] = *bp++;
-#else
-	((char *) &sval)[3] = *bp++;
-	((char *) &sval)[2] = *bp++;
-	((char *) &sval)[1] = *bp++;
-	((char *) &sval)[0] = *bp;
-#endif  /* ! XNTP_BIG_ENDIAN */ 
-	return sval;
-}
-#endif
-
-/*
- * this 'casts' a character array into a double
- */
-double
+static double
 getdbl (
 	u_char *bp
 	)
 {
-	double dval;
-#ifdef WORDS_BIGENDIAN 
-	((char *) &dval)[0] = *bp++;
-	((char *) &dval)[1] = *bp++;
-	((char *) &dval)[2] = *bp++;
-	((char *) &dval)[3] = *bp++;
-	((char *) &dval)[4] = *bp++;
-	((char *) &dval)[5] = *bp++;
-	((char *) &dval)[6] = *bp++;
-	((char *) &dval)[7] = *bp;
+#ifdef WORDS_BIGENDIAN
+	double out;
+
+	memcpy(&out, bp, sizeof(out));
+	return out;
 #else
-	((char *) &dval)[7] = *bp++;
-	((char *) &dval)[6] = *bp++;
-	((char *) &dval)[5] = *bp++;
-	((char *) &dval)[4] = *bp++;
-	((char *) &dval)[3] = *bp++;
-	((char *) &dval)[2] = *bp++;
-	((char *) &dval)[1] = *bp++;
-	((char *) &dval)[0] = *bp;
-#endif  /* ! XNTP_BIG_ENDIAN */ 
-	return dval;
+	union {
+		u_char ch[8];
+		u_int32 u32[2];
+	} ui;
+		
+	union {
+		double out;
+		u_int32 u32[2];
+	} uo;
+
+	memcpy(ui.ch, bp, sizeof(ui.ch));
+	/* least-significant 32 bits of double from swapped bp[4] to bp[7] */
+	uo.u32[0] = ntohl(ui.u32[1]);
+	/* most-significant 32 bits from swapped bp[0] to bp[3] */
+	uo.u32[1] = ntohl(ui.u32[0]);
+
+	return uo.out;
+#endif
 }
 
 /*
- * cast a 16 bit character array into a short (16 bit) int
+ * copy/swap a big-endian palisade short into a host short
  */
-short
+static short
 getint (
 	u_char *bp
 	)
 {
-	return (short) (bp[1] + (bp[0] << 8));
+	u_short us;
+
+	memcpy(&us, bp, sizeof(us));
+	return (short)ntohs(us);
 }
 
 /*
- * cast a 32 bit character array into a long (32 bit) int
+ * copy/swap a big-endian palisade 32-bit int into a host 32-bit int
  */
-long
+static int32
 getlong(
 	u_char *bp
 	)
 {
-	return (long) (bp[0] << 24) | 
-		      (bp[1] << 16) |
-		      (bp[2] << 8) |
-		       bp[3];
+	u_int32 u32;
+
+	memcpy(&u32, bp, sizeof(u32));
+	return (int32)(u_int32)ntohl(u32);
+}
+
+/*
+ * copy/swap a big-endian 32-bit single-precision floating point into a host 32-bit int
+ */
+static int32
+getsingle(
+	u_char *bp
+	)
+{
+	u_int32 mantissa;
+	int8_t exponent;
+	uint8_t sign, exp_field;
+	int32 res;
+
+	memcpy(&mantissa, bp, sizeof(mantissa));
+	mantissa = ((u_int32)ntohl(mantissa) & 0x7FFFFF) | 0x800000;
+	exp_field = ((uint8_t)bp[0] << 1) + ((uint8_t)bp[1] >> 7);
+	exponent = (int8_t)exp_field - 127;
+	sign = ((uint8_t)bp[0] >> 7);
+	if (exponent > 23)
+		res = (int32)(mantissa << (exponent - 23));
+	else
+		res = (int32)(mantissa >> (23 - exponent));
+	return sign ? -res : res;
 }
 
 #else	/* REFCLOCK && CLOCK_PALISADE*/
