@@ -14,13 +14,12 @@
 #include "ntp_fp.h"
 #include "ntp.h"
 #include "ntp_io.h"
-#include "ntp_unixtime.h"
+#include "timevalops.h"
 #include "ntpdate.h"
 #include "ntp_string.h"
 #include "ntp_syslog.h"
 #include "ntp_select.h"
 #include "ntp_stdlib.h"
-#include "ntp_assert.h"
 #include <ssl_applink.c>
 
 #include "isc/net.h"
@@ -110,11 +109,6 @@ static timer_t ntpdate_timerid;
  */
 s_char	sys_precision;		/* local clock precision (log2 s) */
 
-/*
- * Debugging flag
- */
-volatile int debug = 0;
-
 /*
  * File descriptor masks etc. for call to select
  */
@@ -155,12 +149,12 @@ int unpriv_port = 0;
 /*
  * Program name.
  */
-char *progname;
+char const *progname;
 
 /*
  * Systemwide parameters and flags
  */
-int sys_samples = DEFSAMPLES;	/* number of samples/server */
+int sys_samples = 0;		/* number of samples/server, will be modified later */
 u_long sys_timeout = DEFTIMEOUT; /* timeout time, in TIMER_HZ units */
 struct server *sys_servers;	/* the server list */
 int sys_numservers = 0; 	/* number of servers to poll */
@@ -226,7 +220,7 @@ void	input_handler	(void);
 static	int l_adj_systime	(l_fp *);
 static	int l_step_systime	(l_fp *);
 
-static	void	printserver (struct server *, FILE *);
+static	void	print_server (struct server *, FILE *);
 
 #ifdef SYS_WINNT
 int 	on = 1;
@@ -330,7 +324,7 @@ ntpdatemain (
 	key_file = key_file_storage;
 
 	if (!ExpandEnvironmentStrings(KEYFILE, key_file, MAX_PATH))
-		msyslog(LOG_ERR, "ExpandEnvironmentStrings(KEYFILE) failed: %m\n");
+		msyslog(LOG_ERR, "ExpandEnvironmentStrings(KEYFILE) failed: %m");
 
 	ssl_applink();
 #endif /* SYS_WINNT */
@@ -435,7 +429,7 @@ ntpdatemain (
 		default:
 			break;
 	    }
-	
+
 	if (errflg) {
 		(void) fprintf(stderr,
 		    "usage: %s [-46bBdqsuv] [-a key#] [-e delay] [-k file] [-p samples] [-o version#] [-t timeo] server ...\n",
@@ -443,6 +437,14 @@ ntpdatemain (
 		exit(2);
 	}
 
+	/*
+	 * If number of Samples (-p) not specified by user:
+	 * - if a simple_query (-q) just ONE will do
+	 * - otherwise the normal is DEFSAMPLES
+	 */
+	if (sys_samples == 0)
+		 sys_samples = (simple_query ? 1 : DEFSAMPLES);
+
 	if (debug || simple_query) {
 #ifdef HAVE_SETVBUF
 		static char buf[BUFSIZ];
@@ -567,8 +569,8 @@ ntpdatemain (
 			nfound = poll(rdfdes, (unsigned int)nbsock, timeout.tv_sec * 1000);
 
 #else
-			nfound = select(maxfd, &rdfdes, (fd_set *)0,
-					(fd_set *)0, &timeout);
+			nfound = select(maxfd, &rdfdes, NULL, NULL,
+					&timeout);
 #endif
 			if (nfound > 0)
 				input_handler();
@@ -657,9 +659,6 @@ transmit(
 {
 	struct pkt xpkt;
 
-	if (debug)
-		printf("transmit(%s)\n", stoa(&server->srcadr));
-
 	if (server->filter_nextpt < server->xmtcnt) {
 		l_fp ts;
 		/*
@@ -680,6 +679,9 @@ transmit(
 		return;
 	}
 
+	if (debug)
+		printf("transmit(%s)\n", stoa(&server->srcadr));
+
 	/*
 	 * If we're here, send another message to the server.  Fill in
 	 * the packet and let 'er rip.
@@ -702,7 +704,7 @@ transmit(
 	 * If not, just timestamp it and send it away.
 	 */
 	if (sys_authenticate) {
-		int len;
+		size_t len;
 
 		xpkt.exten[0] = htonl(sys_authkey);
 		get_systime(&server->xmt);
@@ -757,7 +759,7 @@ receive(
 	 */
 	if (rbufp->recv_length == LEN_PKT_NOMAC)
 		has_mac = 0;
-	else if (rbufp->recv_length >= LEN_PKT_NOMAC)
+	else if (rbufp->recv_length >= (int)LEN_PKT_NOMAC)
 		has_mac = 1;
 	else {
 		if (debug)
@@ -814,11 +816,11 @@ receive(
 			printf("receive: rpkt keyid=%ld sys_authkey=%ld decrypt=%ld\n",
 			   (long int)ntohl(rpkt->exten[0]), (long int)sys_authkey,
 			   (long int)authdecrypt(sys_authkey, (u_int32 *)rpkt,
-				LEN_PKT_NOMAC, (int)(rbufp->recv_length - LEN_PKT_NOMAC)));
+				LEN_PKT_NOMAC, (size_t)(rbufp->recv_length - LEN_PKT_NOMAC)));
 
 		if (has_mac && ntohl(rpkt->exten[0]) == sys_authkey &&
 			authdecrypt(sys_authkey, (u_int32 *)rpkt, LEN_PKT_NOMAC,
-			(int)(rbufp->recv_length - LEN_PKT_NOMAC)))
+			(size_t)(rbufp->recv_length - LEN_PKT_NOMAC)))
 			is_authentic = 1;
 		if (debug)
 			printf("receive: authentication %s\n",
@@ -834,7 +836,7 @@ receive(
 	if (LEAP_NOTINSYNC == PKT_LEAP(rpkt->li_vn_mode) &&
 	    STRATUM_PKT_UNSPEC == rpkt->stratum &&
 	    !memcmp("RATE", &rpkt->refid, 4)) {
-		msyslog(LOG_ERR, "%s rate limit response from server.\n",
+		msyslog(LOG_ERR, "%s rate limit response from server.",
 			stoa(&rbufp->recv_srcadr));
 		server->event_time = 0;
 		complete_servers++;
@@ -855,7 +857,7 @@ receive(
 	NTOHL_FP(&rpkt->xmt, &server->org);
 
 	/*
-	 * Make sure the server is at least somewhat sane.	If not, try
+	 * Make sure the server is at least somewhat sane. If not, try
 	 * again.
 	 */
 	if (L_ISZERO(&rec) || !L_ISHIS(&server->org, &rec)) {
@@ -961,6 +963,8 @@ clock_filter(
 	register int i, j;
 	int ord[NTP_SHIFT];
 
+	INSIST((0 < sys_samples) && (sys_samples <= NTP_SHIFT));
+
 	/*
 	 * Sort indices into increasing delay order
 	 */
@@ -1046,15 +1050,15 @@ clock_select(void)
 	/*
 	 * This first chunk of code is supposed to go through all
 	 * servers we know about to find the NTP_MAXLIST servers which
-	 * are most likely to succeed.	We run through the list
+	 * are most likely to succeed. We run through the list
 	 * doing the sanity checks and trying to insert anyone who
-	 * looks okay.	We are at all times aware that we should
+	 * looks okay. We are at all times aware that we should
 	 * only keep samples from the top two strata and we only need
 	 * NTP_MAXLIST of them.
 	 */
 	nlist = 0;	/* none yet */
 	for (server = sys_servers; server != NULL; server = server->next_server) {
-		if (server->delay == 0) {
+		if (server->stratum == 0) {
 			if (debug)
 				printf("%s: Server dropped: no data\n", ntoa(&server->srcadr));
 			continue;	/* no data */
@@ -1066,25 +1070,25 @@ clock_select(void)
 		}
 		if (server->delay > NTP_MAXWGT) {
 			if (debug)
-				printf("%s: Server dropped: server too far away\n", 
+				printf("%s: Server dropped: server too far away\n",
 					ntoa(&server->srcadr));
 			continue;	/* too far away */
 		}
 		if (server->leap == LEAP_NOTINSYNC) {
 			if (debug)
-				printf("%s: Server dropped: Leap not in sync\n", ntoa(&server->srcadr));
+				printf("%s: Server dropped: leap not in sync\n", ntoa(&server->srcadr));
 			continue;	/* he's in trouble */
 		}
 		if (!L_ISHIS(&server->org, &server->reftime)) {
 			if (debug)
-				printf("%s: Server dropped: server is very broken\n", 
+				printf("%s: Server dropped: server is very broken\n",
 				       ntoa(&server->srcadr));
 			continue;	/* very broken host */
 		}
 		if ((server->org.l_ui - server->reftime.l_ui)
 		    >= NTP_MAXAGE) {
 			if (debug)
-				printf("%s: Server dropped: Server has gone too long without sync\n", 
+				printf("%s: Server dropped: server has gone too long without sync\n",
 				       ntoa(&server->srcadr));
 			continue;	/* too long without sync */
 		}
@@ -1253,7 +1257,6 @@ static int
 clock_adjust(void)
 {
 	register struct server *sp, *server;
-	s_fp absoffset;
 	int dostep;
 
 	for (sp = sys_servers; sp != NULL; sp = sp->next_server)
@@ -1261,8 +1264,10 @@ clock_adjust(void)
 	server = clock_select();
 
 	if (debug || simple_query) {
+		if (debug)
+			printf ("\n");
 		for (sp = sys_servers; sp != NULL; sp = sp->next_server)
-			printserver(sp, stdout);
+			print_server(sp, stdout);
 	}
 
 	if (server == 0) {
@@ -1276,38 +1281,29 @@ clock_adjust(void)
 	} else if (never_step) {
 		dostep = 0;
 	} else {
-		absoffset = server->soffset;
-		if (absoffset < 0)
-			absoffset = -absoffset;
-		dostep = (absoffset >= NTPDATE_THRESHOLD || absoffset < 0);
+		/* [Bug 3023] get absolute difference, avoiding signed
+		 * integer overflow like hell.
+		 */
+		u_fp absoffset;
+		if (server->soffset < 0)
+			absoffset = 1u + (u_fp)(-(server->soffset + 1));
+		else
+			absoffset = (u_fp)server->soffset;
+		dostep = (absoffset >= NTPDATE_THRESHOLD);
 	}
 
 	if (dostep) {
-		if (simple_query || debug || l_step_systime(&server->offset)){
+		if (simple_query || l_step_systime(&server->offset)){
 			msyslog(LOG_NOTICE, "step time server %s offset %s sec",
 				stoa(&server->srcadr),
 				lfptoa(&server->offset, 6));
 		}
 	} else {
-#ifndef SYS_WINNT
 		if (simple_query || l_adj_systime(&server->offset)) {
 			msyslog(LOG_NOTICE, "adjust time server %s offset %s sec",
 				stoa(&server->srcadr),
 				lfptoa(&server->offset, 6));
 		}
-#else
-		/* The NT SetSystemTimeAdjustment() call achieves slewing by
-		 * changing the clock frequency. This means that we cannot specify
-		 * it to slew the clock by a definite amount and then stop like
-		 * the Unix adjtime() routine. We can technically adjust the clock
-		 * frequency, have ntpdate sleep for a while, and then wake
-		 * up and reset the clock frequency, but this might cause some
-		 * grief if the user attempts to run ntpd immediately after
-		 * ntpdate and the socket is in use.
-		 */
-		printf("\nThe -b option is required by ntpdate on Windows NT platforms\n");
-		exit(1);
-#endif /* SYS_WINNT */
 	}
 	return(0);
 }
@@ -1358,7 +1354,7 @@ addserver(
 	char service[5];
 	sockaddr_u addr;
 
-	strncpy(service, "ntp", sizeof(service));
+	strlcpy(service, "ntp", sizeof(service));
 
 	/* Get host address. Looking for UDP datagram connection. */
 	ZERO(hints);
@@ -1379,13 +1375,13 @@ addserver(
 			   by waiting for resolution of several servers */
 			fprintf(stderr, "Exiting, name server cannot be used: %s (%d)",
 				gai_strerror(error), error);
-			msyslog(LOG_ERR, "name server cannot be used: %s (%d)\n",
+			msyslog(LOG_ERR, "name server cannot be used: %s (%d)",
 				gai_strerror(error), error);
 			exit(1);
 		}
 		fprintf(stderr, "Error resolving %s: %s (%d)\n", serv,
 			gai_strerror(error), error);
-		msyslog(LOG_ERR, "Can't find host %s: %s (%d)\n", serv,
+		msyslog(LOG_ERR, "Can't find host %s: %s (%d)", serv,
 			gai_strerror(error), error);
 		return;
 	}
@@ -1440,7 +1436,7 @@ findserver(
 	if (SRCPORT(addr) != NTP_PORT)
 		return 0;
 
-	for (server = sys_servers; server != NULL; 
+	for (server = sys_servers; server != NULL;
 	     server = server->next_server) {
 		if (SOCK_EQ(addr, &server->srcadr))
 			return server;
@@ -1451,7 +1447,7 @@ findserver(
 		}
 	}
 
-	if (mc_server != NULL) {	
+	if (mc_server != NULL) {
 
 		struct server *sp;
 
@@ -1494,7 +1490,7 @@ timer(void)
 	 * who's event timers have expired.  Give these to
 	 * the transmit routine.
 	 */
-	for (server = sys_servers; server != NULL; 
+	for (server = sys_servers; server != NULL;
 	     server = server->next_server) {
 		if (server->event_time != 0
 		    && server->event_time <= current_time)
@@ -1520,7 +1516,7 @@ alarming(
 	alarm_flag++;
 }
 #else	/* SYS_WINNT follows */
-void CALLBACK 
+void CALLBACK
 alarming(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
 {
 	UNUSED_ARG(uTimerID); UNUSED_ARG(uMsg); UNUSED_ARG(dwUser);
@@ -1605,24 +1601,26 @@ init_alarm(void)
 #else	/* SYS_WINNT follows */
 	_tzset();
 
-	/*
-	 * Get privileges needed for fiddling with the clock
-	 */
+	if (!simple_query && !debug) {
+		/*
+		 * Get privileges needed for fiddling with the clock
+		 */
 
-	/* get the current process token handle */
-	if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) {
-		msyslog(LOG_ERR, "OpenProcessToken failed: %m");
-		exit(1);
+		/* get the current process token handle */
+		if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) {
+			msyslog(LOG_ERR, "OpenProcessToken failed: %m");
+			exit(1);
+		}
+		/* get the LUID for system-time privilege. */
+		LookupPrivilegeValue(NULL, SE_SYSTEMTIME_NAME, &tkp.Privileges[0].Luid);
+		tkp.PrivilegeCount = 1;		/* one privilege to set */
+		tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
+		/* get set-time privilege for this process. */
+		AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,(PTOKEN_PRIVILEGES) NULL, 0);
+		/* cannot test return value of AdjustTokenPrivileges. */
+		if (GetLastError() != ERROR_SUCCESS)
+			msyslog(LOG_ERR, "AdjustTokenPrivileges failed: %m");
 	}
-	/* get the LUID for system-time privilege. */
-	LookupPrivilegeValue(NULL, SE_SYSTEMTIME_NAME, &tkp.Privileges[0].Luid);
-	tkp.PrivilegeCount = 1;		/* one privilege to set */
-	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
-	/* get set-time privilege for this process. */
-	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,(PTOKEN_PRIVILEGES) NULL, 0);
-	/* cannot test return value of AdjustTokenPrivileges. */
-	if (GetLastError() != ERROR_SUCCESS)
-		msyslog(LOG_ERR, "AdjustTokenPrivileges failed: %m");
 
 	/*
 	 * Set up timer interrupts for every 2**EVENT_TIMEOUT seconds
@@ -1695,7 +1693,7 @@ init_io(void)
 	 * Open the socket
 	 */
 
-	strncpy(service, "ntp", sizeof(service));
+	strlcpy(service, "ntp", sizeof(service));
 
 	/*
 	 * Init hints addrinfo structure
@@ -1894,7 +1892,7 @@ input_handler(void)
 #else
 	fd_set fds;
 #endif
-	int fdc = 0;
+	SOCKET fdc = 0;
 
 	/*
 	 * Do a poll to see if we have data
@@ -1918,7 +1916,7 @@ input_handler(void)
 
 #else
 		fds = fdmask;
-		n = select(maxfd, &fds, (fd_set *)0, (fd_set *)0, &tvzero);
+		n = select(maxfd, &fds, NULL, NULL, &tvzero);
 
 		/*
 		 * Determine which socket received data
@@ -1996,7 +1994,6 @@ input_handler(void)
 }
 
 
-#if !defined SYS_WINNT && !defined SYS_CYGWIN32
 /*
  * adj_systime - do a big long slew of the system time
  */
@@ -2041,15 +2038,30 @@ l_adj_systime(
 		adjtv.tv_usec = -adjtv.tv_usec;
 	}
 
-	if (adjtv.tv_usec != 0 && !debug) {
+	if (!debug && (adjtv.tv_usec != 0)) {
+		/* A time correction needs to be applied. */
+#if !defined SYS_WINNT && !defined SYS_CYGWIN32
+		/* Slew the time on systems that support this. */
 		if (adjtime(&adjtv, &oadjtv) < 0) {
 			msyslog(LOG_ERR, "Can't adjust the time of day: %m");
-			return 0;
+			exit(1);
 		}
+#else	/* SYS_WINNT or SYS_CYGWIN32 is defined */
+		/*
+		 * The NT SetSystemTimeAdjustment() call achieves slewing by
+		 * changing the clock frequency. This means that we cannot specify
+		 * it to slew the clock by a definite amount and then stop like
+		 * the Unix adjtime() routine. We can technically adjust the clock
+		 * frequency, have ntpdate sleep for a while, and then wake
+		 * up and reset the clock frequency, but this might cause some
+		 * grief if the user attempts to run ntpd immediately after
+		 * ntpdate and the socket is in use.
+		 */
+		printf("\nSlewing the system time is not supported on Windows. Use the -b option to step the time.\n");
+#endif	/* defined SYS_WINNT || defined SYS_CYGWIN32 */
 	}
 	return 1;
 }
-#endif /* SYS_WINNT */
 
 
 /*
@@ -2068,11 +2080,14 @@ l_step_systime(
 	int isneg;
 	int n;
 
-	if (debug) return 1;
+	if (debug)
+		return 1;
+
 	/*
 	 * Take the absolute value of the offset
 	 */
 	ftmp = *ts;
+
 	if (L_ISNEG(&ftmp)) {
 		L_NEG(&ftmp);
 		isneg = 1;
@@ -2082,9 +2097,9 @@ l_step_systime(
 	if (ftmp.l_ui >= 3) {		/* Step it and slew - we might win */
 		LFPTOD(ts, dtemp);
 		n = step_systime(dtemp);
-		if (!n)
-			return n;
-		if (isneg)
+		if (n == 0)
+			return 0;
+		if (isneg)		/* WTF! */
 			ts->l_ui = ~0;
 		else
 			ts->l_ui = ~0;
@@ -2113,12 +2128,12 @@ l_step_systime(
 }
 
 
-/* XXX ELIMINATE printserver similar in ntptrace.c, ntpdate.c */
+/* XXX ELIMINATE print_server similar in ntptrace.c, ntpdate.c */
 /*
- * printserver - print detail information for a server
+ * print_server - print detail information for a server
  */
 static void
-printserver(
+print_server(
 	register struct server *pp,
 	FILE *fp
 	)
@@ -2127,6 +2142,9 @@ printserver(
 	char junk[5];
 	const char *str;
 
+	if (pp->stratum == 0)		/* Nothing received => nothing to print */
+		return;
+
 	if (!debug) {
 		(void) fprintf(fp, "server %s, stratum %d, offset %s, delay %s\n",
 				   stoa(&pp->srcadr), pp->stratum,
@@ -2143,17 +2161,20 @@ printserver(
 			   pp->leap & 0x1 ? '1' : '0',
 			   pp->trust);
 
-	if (pp->stratum == 1) {
-		junk[4] = 0;
-		memmove(junk, (char *)&pp->refid, 4);
+	if (REFID_ISTEXT(pp->stratum)) {
+		str = (char *) &pp->refid;
+		for (i=0; i<4 && str[i]; i++) {
+			junk[i] = (isprint(str[i]) ? str[i] : '.');
+		}
+		junk[i] = 0; // force terminating 0
 		str = junk;
 	} else {
-		str = stoa(&pp->srcadr);
+		str = numtoa(pp->refid);
 	}
 	(void) fprintf(fp,
-			   "refid [%s], delay %s, dispersion %s\n",
-			   str, fptoa((s_fp)pp->delay, 5),
-			   ufptoa(pp->dispersion, 5));
+			"refid [%s], root delay %s, root dispersion %s\n",
+			str, fptoa((s_fp)pp->rootdelay, 6),
+			ufptoa(pp->rootdisp, 6));
 
 	(void) fprintf(fp, "transmitted %d, in filter %d\n",
 			   pp->xmtcnt, pp->filter_nextpt);
@@ -2165,21 +2186,23 @@ printserver(
 	(void) fprintf(fp, "transmit timestamp:  %s\n",
 			   prettydate(&pp->xmt));
 
-	(void) fprintf(fp, "filter delay: ");
-	for (i = 0; i < NTP_SHIFT; i++) {
-		(void) fprintf(fp, " %-8.8s", fptoa(pp->filter_delay[i], 5));
-		if (i == (NTP_SHIFT>>1)-1)
-			(void) fprintf(fp, "\n        ");
-	}
-	(void) fprintf(fp, "\n");
+	if (sys_samples > 1) {
+		(void) fprintf(fp, "filter delay: ");
+		for (i = 0; i < NTP_SHIFT; i++) {
+			(void) fprintf(fp, " %-8.8s", fptoa(pp->filter_delay[i], 5));
+			if (i == (NTP_SHIFT>>1)-1)
+				(void) fprintf(fp, "\n        ");
+		}
+		(void) fprintf(fp, "\n");
 
-	(void) fprintf(fp, "filter offset:");
-	for (i = 0; i < PEER_SHIFT; i++) {
-		(void) fprintf(fp, " %-8.8s", lfptoa(&pp->filter_offset[i], 6));
-		if (i == (PEER_SHIFT>>1)-1)
-			(void) fprintf(fp, "\n        ");
+		(void) fprintf(fp, "filter offset:");
+		for (i = 0; i < PEER_SHIFT; i++) {
+			(void) fprintf(fp, " %-8.8s", lfptoa(&pp->filter_offset[i], 6));
+			if (i == (PEER_SHIFT>>1)-1)
+				(void) fprintf(fp, "\n        ");
+		}
+		(void) fprintf(fp, "\n");
 	}
-	(void) fprintf(fp, "\n");
 
 	(void) fprintf(fp, "delay %s, dispersion %s\n",
 			   fptoa((s_fp)pp->delay, 5), ufptoa(pp->dispersion, 5));
@@ -2227,7 +2250,7 @@ isc_boolean_t ntp_port_inuse(int af, u_short port)
 	 * Check if NTP socket is already in use on this system
 	 * This is only for Windows Systems, as they tend not to fail on the real bind() below
 	 */
-	
+
 	SOCKET checksocket;
 	struct sockaddr_in checkservice;
 	checksocket = socket(af, SOCK_DGRAM, 0);
