@@ -1,30 +1,38 @@
 /*
  * ntpdc - control and monitor your ntpd daemon
  */
-
+#include <config.h>
 #include <stdio.h>
 #include <stddef.h>
 #include <ctype.h>
 #include <signal.h>
 #include <setjmp.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+#ifdef SYS_WINNT
+# include <mswsock.h>
+#endif
+#include <isc/net.h>
+#include <isc/result.h>
 
 #include "ntpdc.h"
 #include "ntp_select.h"
-#include "ntp_io.h"
 #include "ntp_stdlib.h"
 #include "ntp_assert.h"
 #include "ntp_lineedit.h"
-#include "isc/net.h"
-#include "isc/result.h"
+#ifdef OPENSSL
+#include "openssl/evp.h"
+#include "openssl/objects.h"
+#endif
 #include <ssl_applink.c>
 
 #include "ntp_libopts.h"
 #include "ntpdc-opts.h"
-
-#ifdef SYS_WINNT
-# include <Mswsock.h>
-# include <io.h>
-#endif /* SYS_WINNT */
+#include "safecast.h"
 
 #ifdef SYS_VXWORKS
 				/* vxWorks needs mode flag -casey*/
@@ -67,8 +75,8 @@ int		ntpdcmain	(int,	char **);
 static	int	openhost	(const char *);
 static	int	sendpkt		(void *, size_t);
 static	void	growpktdata	(void);
-static	int	getresponse	(int, int, int *, int *, char **, int);
-static	int	sendrequest	(int, int, int, u_int, size_t, char *);
+static	int	getresponse	(int, int, size_t *, size_t *, const char **, size_t);
+static	int	sendrequest	(int, int, int, size_t, size_t, const char *);
 static	void	getcmds		(void);
 static	RETSIGTYPE abortcmd	(int);
 static	void	docmd		(const char *);
@@ -89,8 +97,10 @@ static	void	hostnames	(struct parse *, FILE *);
 static	void	setdebug	(struct parse *, FILE *);
 static	void	quit		(struct parse *, FILE *);
 static	void	version		(struct parse *, FILE *);
-static	void	warning		(const char *, const char *, const char *);
-static	void	error		(const char *, const char *, const char *);
+static	void	warning		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
+static	void	error		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
 static	u_long	getkeyid	(const char *);
 
 
@@ -216,28 +226,34 @@ static	const char *chosts[MAXHOSTS];
 #define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)
 
 /*
- * For converting time stamps to dates
+ * Jump buffer for longjumping back to the command level.
+ *
+ * See ntpq/ntpq.c for an explanation why 'sig{set,long}jmp()' is used
+ * when available.
  */
-#define	JAN_1970	2208988800	/* 1970 - 1900 in seconds */
-
-/*
- * Jump buffer for longjumping back to the command level
- */
-static	jmp_buf interrupt_buf;
-static  volatile int jump = 0;
+#if HAVE_DECL_SIGSETJMP && HAVE_DECL_SIGLONGJMP
+# define JMP_BUF	sigjmp_buf
+# define SETJMP(x)	sigsetjmp((x), 1)
+# define LONGJMP(x, v)	siglongjmp((x),(v))
+#else
+# define JMP_BUF	jmp_buf
+# define SETJMP(x)	setjmp((x))
+# define LONGJMP(x, v)	longjmp((x),(v))
+#endif
+static	JMP_BUF		interrupt_buf;
+static	volatile int	jump = 0;
 
 /*
  * Pointer to current output unit
  */
-static	FILE *current_output;
+static	FILE *current_output = NULL;
 
 /*
  * Command table imported from ntpdc_ops.c
  */
 extern struct xcmd opcmds[];
 
-char *progname;
-volatile int debug;
+char const *progname;
 
 #ifdef NO_MAIN_ALLOWED
 CALL(ntpdc,"ntpdc",ntpdcmain);
@@ -271,8 +287,6 @@ ntpdcmain(
 	char *argv[]
 	)
 {
-	extern int ntp_optind;
-
 	delay_time.l_ui = 0;
 	delay_time.l_uf = DEFDELAY;
 
@@ -283,6 +297,7 @@ ntpdcmain(
 
 	init_lib();	/* sets up ipv4_works, ipv6_works */
 	ssl_applink();
+	init_auth();
 
 	/* Check to see if we have IPv6. Otherwise default to IPv4 */
 	if (!ipv6_works)
@@ -312,7 +327,7 @@ ntpdcmain(
 		}
 	}
 
-	debug = DESC(DEBUG_LEVEL).optOccCt;
+	debug = OPT_VALUE_SET_DEBUG_LEVEL;
 
 	if (HAVE_OPT(INTERACTIVE)) {
 		interactive = 1;
@@ -346,65 +361,9 @@ ntpdcmain(
 		interactive = 1;
 	}
 
-#if 0
-	ai_fam_templ = ai_fam_default;
-	while ((c = ntp_getopt(argc, argv, "46c:dilnps")) != EOF)
-	    switch (c) {
-		case '4':
-		    ai_fam_templ = AF_INET;
-		    break;
-		case '6':
-		    ai_fam_templ = AF_INET6;
-		    break;
-		case 'c':
-		    ADDCMD(ntp_optarg);
-		    break;
-		case 'd':
-		    ++debug;
-		    break;
-		case 'i':
-		    interactive = 1;
-		    break;
-		case 'l':
-		    ADDCMD("listpeers");
-		    break;
-		case 'n':
-		    showhostnames = 0;
-		    break;
-		case 'p':
-		    ADDCMD("peers");
-		    break;
-		case 's':
-		    ADDCMD("dmpeers");
-		    break;
-		default:
-		    errflg++;
-		    break;
-	    }
-
-	if (errflg) {
-		(void) fprintf(stderr,
-			       "usage: %s [-46dilnps] [-c cmd] host ...\n",
-			       progname);
-		exit(2);
-	}
-
-	if (ntp_optind == argc) {
-		ADDHOST(DEFHOST);
-	} else {
-		for (; ntp_optind < argc; ntp_optind++)
-		    ADDHOST(argv[ntp_optind]);
-	}
-
-	if (numcmds == 0 && interactive == 0
-	    && isatty(fileno(stdin)) && isatty(fileno(stderr))) {
-		interactive = 1;
-	}
-#endif
-
 #ifndef SYS_WINNT /* Under NT cannot handle SIGINT, WIN32 spawns a handler */
 	if (interactive)
-	    (void) signal_no_reset(SIGINT, abortcmd);
+		(void) signal_no_reset(SIGINT, abortcmd);
 #endif /* SYS_WINNT */
 
 	/*
@@ -445,29 +404,28 @@ openhost(
 	)
 {
 	char temphost[LENHOSTNAME];
-	int a_info, i;
+	int a_info;
 	struct addrinfo hints, *ai = NULL;
-	register const char *cp;
+	sockaddr_u addr;
+	size_t octets;
+	const char *cp;
 	char name[LENHOSTNAME];
 	char service[5];
 
 	/*
 	 * We need to get by the [] if they were entered 
 	 */
-	
-	cp = hname;
-	
-	if (*cp == '[') {
-		cp++;	
-		for (i = 0; *cp && *cp != ']'; cp++, i++)
-			name[i] = *cp;
-		if (*cp == ']') {
-			name[i] = '\0';
-			hname = name;
-		} else {
+	if (*hname == '[') {
+		cp = strchr(hname + 1, ']');
+		if (!cp || (octets = (size_t)(cp - hname) - 1) >= sizeof(name)) {
+			errno = EINVAL;
+			warning("%s", "bad hostname/address");
 			return 0;
 		}
-	}	
+		memcpy(name, hname + 1, octets);
+		name[octets] = '\0';
+		hname = name;
+	}
 
 	/*
 	 * First try to resolve it as an ip address and if that fails,
@@ -476,8 +434,8 @@ openhost(
 	 * will return an "IPv4-mapped IPv6 address" address if you
 	 * give it an IPv4 address to lookup.
 	 */
-	strcpy(service, "ntp");
-	memset((char *)&hints, 0, sizeof(struct addrinfo));
+	strlcpy(service, "ntp", sizeof(service));
+	ZERO(hints);
 	hints.ai_family = ai_fam_templ;
 	hints.ai_protocol = IPPROTO_UDP;
 	hints.ai_socktype = SOCK_DGRAM;
@@ -501,7 +459,7 @@ openhost(
 		a_info = getaddrinfo(hname, service, &hints, &ai);	
 	}
 	if (a_info != 0) {
-		(void) fprintf(stderr, "%s\n", gai_strerror(a_info));
+		fprintf(stderr, "%s\n", gai_strerror(a_info));
 		if (ai != NULL)
 			freeaddrinfo(ai);
 		return 0;
@@ -511,30 +469,29 @@ openhost(
 	 * getaddrinfo() has returned without error so ai should not 
 	 * be NULL.
 	 */
-	NTP_INSIST(ai != NULL);
+	INSIST(ai != NULL);
+	ZERO(addr);
+	octets = min(sizeof(addr), ai->ai_addrlen);
+	memcpy(&addr, ai->ai_addr, octets);
 
-	if (ai->ai_canonname == NULL) {
-		strncpy(temphost, stoa((sockaddr_u *)ai->ai_addr),
-		    LENHOSTNAME);
-		temphost[LENHOSTNAME-1] = '\0';
-	} else {
-		strncpy(temphost, ai->ai_canonname, LENHOSTNAME);
-		temphost[LENHOSTNAME-1] = '\0';
-	}
+	if (ai->ai_canonname == NULL)
+		strlcpy(temphost, stoa(&addr), sizeof(temphost));
+	else
+		strlcpy(temphost, ai->ai_canonname, sizeof(temphost));
 
 	if (debug > 2)
-	    printf("Opening host %s\n", temphost);
+		printf("Opening host %s\n", temphost);
 
 	if (havehost == 1) {
 		if (debug > 2)
-		    printf("Closing old host %s\n", currenthost);
-		(void) closesocket(sockfd);
+			printf("Closing old host %s\n", currenthost);
+		closesocket(sockfd);
 		havehost = 0;
 	}
-	(void) strcpy(currenthost, temphost);
+	strlcpy(currenthost, temphost, sizeof(currenthost));
 	
 	/* port maps to the same in both families */
-	s_port = ((struct sockaddr_in6 *)ai->ai_addr)->sin6_port; 
+	s_port = NSRCPORT(&addr);; 
 #ifdef SYS_VXWORKS
 	((struct sockaddr_in6 *)&hostaddr)->sin6_port = htons(SERVER_PORT_NUM);
 	if (ai->ai_family == AF_INET)
@@ -550,24 +507,19 @@ openhost(
 		int optionValue = SO_SYNCHRONOUS_NONALERT;
 		int err;
 
-		err = setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (char *)&optionValue, sizeof(optionValue));
+		err = setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (void *)&optionValue, sizeof(optionValue));
 		if (err != NO_ERROR) {
 			(void) fprintf(stderr, "cannot open nonoverlapped sockets\n");
 			exit(1);
 		}
 	}
+#endif /* SYS_WINNT */
 
 	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
 	if (sockfd == INVALID_SOCKET) {
-		error("socket", "", "");
+		error("socket");
 		exit(-1);
 	}
-#else
-	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
-	if (sockfd == -1)
-	    error("socket", "", "");
-#endif /* SYS_WINNT */
-
 	
 #ifdef NEED_RCVBUF_SLOP
 # ifdef SO_RCVBUF
@@ -575,8 +527,8 @@ openhost(
 		int rbufsize = INITDATASIZE + 2048; /* 2K for slop */
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF,
-			       &rbufsize, sizeof(int)) == -1)
-		    error("setsockopt", "", "");
+			       (void *)&rbufsize, sizeof(int)) == -1)
+		    error("setsockopt");
 	}
 # endif
 #endif
@@ -585,10 +537,12 @@ openhost(
 	if (connect(sockfd, (struct sockaddr *)&hostaddr, 
 		    sizeof(hostaddr)) == -1)
 #else
-	if (connect(sockfd, (struct sockaddr *)ai->ai_addr,
-		    ai->ai_addrlen) == -1)
+	if (connect(sockfd, ai->ai_addr, ai->ai_addrlen) == -1)
 #endif /* SYS_VXWORKS */
-	    error("connect", "", "");
+	{
+		error("connect");
+		exit(-1);
+	}
 
 	freeaddrinfo(ai);
 	havehost = 1;
@@ -609,7 +563,7 @@ sendpkt(
 	)
 {
 	if (send(sockfd, xdata, xdatalen, 0) == -1) {
-		warning("write to %s failed", currenthost, "");
+		warning("write to %s failed", currenthost);
 		return -1;
 	}
 
@@ -623,8 +577,11 @@ sendpkt(
 static void
 growpktdata(void)
 {
+	size_t priorsz;
+
+	priorsz = (size_t)pktdatasize;
 	pktdatasize += INCDATASIZE;
-	pktdata = erealloc(pktdata, (size_t)pktdatasize);
+	pktdata = erealloc_zero(pktdata, (size_t)pktdatasize, priorsz);
 }
 
 
@@ -635,18 +592,18 @@ static int
 getresponse(
 	int implcode,
 	int reqcode,
-	int *ritems,
-	int *rsize,
-	char **rdata,
-	int esize
+	size_t *ritems,
+	size_t *rsize,
+	const char **rdata,
+	size_t esize
 	)
 {
 	struct resp_pkt rpkt;
 	struct sock_timeval tvo;
-	int items;
-	int i;
-	int size;
-	int datasize;
+	size_t items;
+	size_t i;
+	size_t size;
+	size_t datasize;
 	char *datap;
 	char *tmp_data;
 	char haveseq[MAXSEQ+1];
@@ -655,8 +612,12 @@ getresponse(
 	int numrecv;
 	int seq;
 	fd_set fds;
-	int n;
+	ssize_t n;
 	int pad;
+	/* absolute timeout checks. Not 'time_t' by intention! */
+	uint32_t tobase;	/* base value for timeout */
+	uint32_t tospan;	/* timeout span (max delay) */
+	uint32_t todiff;	/* current delay */
 
 	/*
 	 * This is pretty tricky.  We may get between 1 and many packets
@@ -671,29 +632,42 @@ getresponse(
 	numrecv = 0;
 	*rdata = datap = pktdata;
 	lastseq = 999;	/* too big to be a sequence number */
-	memset(haveseq, 0, sizeof(haveseq));
+	ZERO(haveseq);
 	FD_ZERO(&fds);
+	tobase = (uint32_t)time(NULL);
 
     again:
 	if (firstpkt)
 		tvo = tvout;
 	else
 		tvo = tvsout;
+	tospan = (uint32_t)tvo.tv_sec + (tvo.tv_usec != 0);
 	
 	FD_SET(sockfd, &fds);
-	n = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvo);
-
+	n = select(sockfd+1, &fds, NULL, NULL, &tvo);
 	if (n == -1) {
-		warning("select fails", "", "");
+		warning("select fails");
 		return -1;
 	}
+	
+	/*
+	 * Check if this is already too late. Trash the data and fake a
+	 * timeout if this is so.
+	 */
+	todiff = (((uint32_t)time(NULL)) - tobase) & 0x7FFFFFFFu;
+	if ((n > 0) && (todiff > tospan)) {
+		n = recv(sockfd, (char *)&rpkt, sizeof(rpkt), 0);
+		n -= n; /* faked timeout return from 'select()'*/
+	}
+	
 	if (n == 0) {
 		/*
 		 * Timed out.  Return what we have
 		 */
 		if (firstpkt) {
 			(void) fprintf(stderr,
-				       "%s: timed out, nothing received\n", currenthost);
+				       "%s: timed out, nothing received\n",
+				       currenthost);
 			return ERR_TIMEOUT;
 		} else {
 			(void) fprintf(stderr,
@@ -703,7 +677,7 @@ getresponse(
 				printf("Received sequence numbers");
 				for (n = 0; n <= MAXSEQ; n++)
 				    if (haveseq[n])
-					printf(" %d,", n);
+					printf(" %zd,", (size_t)n);
 				if (lastseq != 999)
 				    printf(" last frame received\n");
 				else
@@ -715,7 +689,7 @@ getresponse(
 
 	n = recv(sockfd, (char *)&rpkt, sizeof(rpkt), 0);
 	if (n == -1) {
-		warning("read", "", "");
+		warning("read");
 		return -1;
 	}
 
@@ -723,37 +697,37 @@ getresponse(
 	/*
 	 * Check for format errors.  Bug proofing.
 	 */
-	if (n < RESP_HEADER_SIZE) {
+	if (n < (ssize_t)RESP_HEADER_SIZE) {
 		if (debug)
-		    printf("Short (%d byte) packet received\n", n);
+			printf("Short (%zd byte) packet received\n", (size_t)n);
 		goto again;
 	}
 	if (INFO_VERSION(rpkt.rm_vn_mode) > NTP_VERSION ||
 	    INFO_VERSION(rpkt.rm_vn_mode) < NTP_OLDVERSION) {
 		if (debug)
-		    printf("Packet received with version %d\n",
-			   INFO_VERSION(rpkt.rm_vn_mode));
+			printf("Packet received with version %d\n",
+			       INFO_VERSION(rpkt.rm_vn_mode));
 		goto again;
 	}
 	if (INFO_MODE(rpkt.rm_vn_mode) != MODE_PRIVATE) {
 		if (debug)
-		    printf("Packet received with mode %d\n",
-			   INFO_MODE(rpkt.rm_vn_mode));
+			printf("Packet received with mode %d\n",
+			       INFO_MODE(rpkt.rm_vn_mode));
 		goto again;
 	}
 	if (INFO_IS_AUTH(rpkt.auth_seq)) {
 		if (debug)
-		    printf("Encrypted packet received\n");
+			printf("Encrypted packet received\n");
 		goto again;
 	}
 	if (!ISRESPONSE(rpkt.rm_vn_mode)) {
 		if (debug)
-		    printf("Received request packet, wanted response\n");
+			printf("Received request packet, wanted response\n");
 		goto again;
 	}
 	if (INFO_MBZ(rpkt.mbz_itemsize) != 0) {
 		if (debug)
-		    printf("Received packet with nonzero MBZ field!\n");
+			printf("Received packet with nonzero MBZ field!\n");
 		goto again;
 	}
 
@@ -762,7 +736,7 @@ getresponse(
 	 */
 	if (rpkt.implementation != implcode || rpkt.request != reqcode) {
 		if (debug)
-		    printf(
+			printf(
 			    "Received implementation/request of %d/%d, wanted %d/%d",
 			    rpkt.implementation, rpkt.request,
 			    implcode, reqcode);
@@ -793,8 +767,8 @@ getresponse(
 	if ((size_t)datasize > (n-RESP_HEADER_SIZE)) {
 		if (debug)
 		    printf(
-			    "Received items %d, size %d (total %d), data in packet is %lu\n",
-			    items, size, datasize, (u_long)(n-RESP_HEADER_SIZE));
+			    "Received items %zu, size %zu (total %zu), data in packet is %zu\n",
+			    items, size, datasize, n-RESP_HEADER_SIZE);
 		goto again;
 	}
 
@@ -802,9 +776,9 @@ getresponse(
 	 * If this isn't our first packet, make sure the size matches
 	 * the other ones.
 	 */
-	if (!firstpkt && esize != *rsize) {
+	if (!firstpkt && size != *rsize) {
 		if (debug)
-		    printf("Received itemsize %d, previous %d\n",
+		    printf("Received itemsize %zu, previous %zu\n",
 			   size, *rsize);
 		goto again;
 	}
@@ -831,10 +805,12 @@ getresponse(
 	}
 
 	/*
-	 * So far, so good.  Copy this data into the output array.
+	 * So far, so good.  Copy this data into the output array. Bump
+	 * the timeout base, in case we expect more data.
 	 */
+	tobase = (uint32_t)time(NULL);
 	if ((datap + datasize + (pad * items)) > (pktdata + pktdatasize)) {
-		int offset = datap - pktdata;
+		size_t offset = datap - pktdata;
 		growpktdata();
 		*rdata = pktdata; /* might have been realloced ! */
 		datap = pktdata + offset;
@@ -844,11 +820,11 @@ getresponse(
 	 * items.  This is so we can play nice with older implementations
 	 */
 
-	tmp_data = rpkt.data;
+	tmp_data = rpkt.u.data;
 	for (i = 0; i < items; i++) {
 		memcpy(datap, tmp_data, (unsigned)size);
 		tmp_data += size;
-		memset(datap + size, 0, pad);
+		zero_mem(datap + size, pad);
 		datap += size + pad;
 	}
 
@@ -897,9 +873,9 @@ sendrequest(
 	int implcode,
 	int reqcode,
 	int auth,
-	u_int qitems,
+	size_t qitems,
 	size_t qsize,
-	char *qdata
+	const char *qdata
 	)
 {
 	struct req_pkt qpkt;
@@ -908,18 +884,17 @@ sendrequest(
 	u_long	key_id;
 	l_fp	ts;
 	l_fp *	ptstamp;
-	int	maclen;
+	size_t	maclen;
 	char *	pass;
 
-	memset(&qpkt, 0, sizeof(qpkt));
-
+	ZERO(qpkt);
 	qpkt.rm_vn_mode = RM_VN_MODE(0, 0, 0);
 	qpkt.implementation = (u_char)implcode;
 	qpkt.request = (u_char)reqcode;
 
 	datasize = qitems * qsize;
 	if (datasize && qdata != NULL) {
-		memcpy(qpkt.data, qdata, datasize);
+		memcpy(qpkt.u.data, qdata, datasize);
 		qpkt.err_nitems = ERR_NITEMS(0, qitems);
 		qpkt.mbz_itemsize = MBZ_ITEMSIZE(qsize);
 	} else {
@@ -972,15 +947,16 @@ sendrequest(
 	get_systime(&ts);
 	L_ADD(&ts, &delay_time);
 	HTONL_FP(&ts, ptstamp);
-	maclen = authencrypt(info_auth_keyid, (void *)&qpkt, reqsize);
+	maclen = authencrypt(
+		info_auth_keyid, (void *)&qpkt, size2int_chk(reqsize));
 	if (!maclen) {  
 		fprintf(stderr, "Key not found\n");
 		return 1;
-	} else if (maclen != (info_auth_hashlen + sizeof(keyid_t))) {
+	} else if (maclen != (size_t)(info_auth_hashlen + sizeof(keyid_t))) {
 		fprintf(stderr,
-			"%d octet MAC, %lu expected with %lu octet digest\n",
-			maclen, (u_long)(info_auth_hashlen + sizeof(keyid_t)),
-			(u_long)info_auth_hashlen);
+			"%zu octet MAC, %zu expected with %zu octet digest\n",
+			maclen, (info_auth_hashlen + sizeof(keyid_t)),
+			info_auth_hashlen);
 		return 1;
 	}
 	return sendpkt(&qpkt, reqsize + maclen);
@@ -995,12 +971,12 @@ doquery(
 	int implcode,
 	int reqcode,
 	int auth,
-	int qitems,
-	int qsize,
-	char *qdata,
-	int *ritems,
-	int *rsize,
-	char **rdata,
+	size_t qitems,
+	size_t qsize,
+	const char *qdata,
+	size_t *ritems,
+	size_t *rsize,
+	const char **rdata,
  	int quiet_mask,
 	int esize
 	)
@@ -1026,10 +1002,9 @@ again:
 		tvzero.tv_sec = tvzero.tv_usec = 0;
 		FD_ZERO(&fds);
 		FD_SET(sockfd, &fds);
-		res = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvzero);
-
+		res = select(sockfd+1, &fds, NULL, NULL, &tvzero);
 		if (res == -1) {
-			warning("polling select", "", "");
+			warning("polling select");
 			return -1;
 		} else if (res > 0)
 
@@ -1085,7 +1060,7 @@ again:
 			if (implcode == IMPL_XNTPD)
 				break;
 			(void) fprintf(stderr,
-				       "***Server implementation incompatable with our own\n");
+				       "***Server implementation incompatible with our own\n");
 			break;
 		case INFO_ERR_REQ:
 			(void) fprintf(stderr,
@@ -1151,12 +1126,14 @@ abortcmd(
 	int sig
 	)
 {
-
 	if (current_output == stdout)
-	    (void) fflush(stdout);
+		(void)fflush(stdout);
 	putc('\n', stderr);
-	(void) fflush(stderr);
-	if (jump) longjmp(interrupt_buf, 1);
+	(void)fflush(stderr);
+	if (jump) {
+		jump = 0;
+		LONGJMP(interrupt_buf, 1);
+	}
 }
 #endif /* SYS_WINNT */
 
@@ -1179,6 +1156,11 @@ docmd(
 	/*
 	 * Tokenize the command line.  If nothing on it, return.
 	 */
+	if (strlen(cmdline) >= MAXLINE) {
+		fprintf(stderr, "***Command ignored, more than %d characters:\n%s\n",
+			MAXLINE - 1, cmdline);
+		return;
+	}
 	tokenize(cmdline, tokens, &ntok);
 	if (ntok == 0)
 	    return;
@@ -1263,14 +1245,22 @@ docmd(
 		current_output = stdout;
 	}
 
-	if (interactive && setjmp(interrupt_buf)) {
-		return;
+	if (interactive) {
+		if ( ! SETJMP(interrupt_buf)) {
+			jump = 1;
+			(xcmd->handler)(&pcmd, current_output);
+			jump = 0;
+		} else {
+			fflush(current_output);
+			fputs("\n >>> command aborted <<<\n", stderr);
+			fflush(stderr);
+		}
 	} else {
-		jump = 1;
-		(xcmd->handler)(&pcmd, current_output);
 		jump = 0;
-		if (current_output != stdout)
-			(void) fclose(current_output);
+		(xcmd->handler)(&pcmd, current_output);
+	}
+	if ((NULL != current_output) && (stdout != current_output)) {
+		(void)fclose(current_output);
 		current_output = NULL;
 	}
 }
@@ -1320,7 +1310,7 @@ findcmd(
 	)
 {
 	register struct xcmd *cl;
-	register int clen;
+	size_t clen;
 	int nmatch;
 	struct xcmd *nearmatch = NULL;
 	struct xcmd *clist;
@@ -1396,8 +1386,7 @@ getarg(
 	char *cp, *np;
 	static const char *digits = "0123456789";
 
-	memset(argp, 0, sizeof(*argp));
-
+	ZERO(*argp);
 	argp->string = str;
 	argp->type   = code & ~OPT;
 
@@ -1434,7 +1423,7 @@ getarg(
 				return 0;
 			}
 			argp->uval *= 10;
-			argp->uval += (cp - digits);
+			argp->uval += (u_long)(cp - digits);
 		} while (*(++np) != '\0');
 
 		if (isneg) {
@@ -1493,18 +1482,16 @@ getnetnum(
 				    LENHOSTNAME, NULL, 0, 0);
 		return 1;
 	} else if (getaddrinfo(hname, "ntp", &hints, &ai) == 0) {
-		NTP_INSIST(sizeof(*num) >= ai->ai_addrlen);
+		INSIST(sizeof(*num) >= ai->ai_addrlen);
 		memcpy(num, ai->ai_addr, ai->ai_addrlen);
 		if (fullhost != NULL) {
-			if (ai->ai_canonname != NULL) {
-				strncpy(fullhost, ai->ai_canonname,
+			if (ai->ai_canonname != NULL)
+				strlcpy(fullhost, ai->ai_canonname,
 					LENHOSTNAME);
-				fullhost[LENHOSTNAME - 1] = '\0';
-			} else {
+			else
 				getnameinfo(&num->sa, SOCKLEN(num),
 					    fullhost, LENHOSTNAME, NULL,
 					    0, 0);
-			}
 		}
 		return 1;
 	}
@@ -1513,21 +1500,22 @@ getnetnum(
 	return 0;
 }
 
+
 /*
  * nntohost - convert network number to host name.  This routine enforces
  *	       the showhostnames setting.
  */
-char *
+const char *
 nntohost(
 	sockaddr_u *netnum
 	)
 {
-	if (!showhostnames)
+	if (!showhostnames || SOCK_UNSPEC(netnum))
 		return stoa(netnum);
-
-	if (ISREFCLOCKADR(netnum))
+	else if (ISREFCLOCKADR(netnum))
 		return refnumtoa(netnum);
-	return socktohost(netnum);
+	else
+		return socktohost(netnum);
 }
 
 
@@ -1561,8 +1549,7 @@ help(
 		for (xcp = opcmds; xcp->keyword != 0; xcp++)
 			list[words++] = xcp->keyword;
 
-		qsort((void *)list, (size_t)words, sizeof(list[0]),
-		      helpsort);
+		qsort((void *)list, words, sizeof(list[0]), helpsort);
 		col = 0;
 		for (word = 0; word < words; word++) {
 			length = strlen(list[word]);
@@ -1576,7 +1563,8 @@ help(
 
 		for (row = 0; row < rows; row++) {
 			for (word = row; word < words; word += rows)
-				fprintf(fp, "%-*.*s", col, col-1, list[word]);
+				fprintf(fp, "%-*.*s", (int)col,
+					(int)col - 1, list[word]);
 			fprintf(fp, "\n");
 		}
 	} else {
@@ -1824,20 +1812,17 @@ passwd(
 			return;
 		}
 	}
-	if (!interactive) {
-		authusekey(info_auth_keyid, info_auth_keytype,
-			   (u_char *)pcmd->argval[0].string);
-		authtrust(info_auth_keyid, 1);
-	} else {
+	if (pcmd->nargs >= 1)
+		pass = pcmd->argval[0].string;
+	else {
 		pass = getpass_keytype(info_auth_keytype);
-		if (*pass == '\0')
-		    (void) fprintf(fp, "Password unchanged\n");
-		else {
-		    authusekey(info_auth_keyid, info_auth_keytype,
-			       (u_char *)pass);
-		    authtrust(info_auth_keyid, 1);
+		if ('\0' == *pass) {
+			fprintf(fp, "Password unchanged\n");
+			return;
 		}
 	}
+	authusekey(info_auth_keyid, info_auth_keytype, (u_char *)pass);
+	authtrust(info_auth_keyid, 1);
 }
 
 
@@ -1924,34 +1909,44 @@ version(
 }
 
 
+static void __attribute__((__format__(__printf__, 1, 0)))
+vwarning(const char *fmt, va_list ap)
+{
+	int serrno = errno;
+	(void) fprintf(stderr, "%s: ", progname);
+	vfprintf(stderr, fmt, ap);
+	(void) fprintf(stderr, ": %s\n", strerror(serrno));
+}
+
 /*
  * warning - print a warning message
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 warning(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	(void) fprintf(stderr, "%s: ", progname);
-	(void) fprintf(stderr, fmt, st1, st2);
-	(void) fprintf(stderr, ": ");
-	perror("");
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 }
 
 
 /*
  * error - print a message and exit
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 error(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	warning(fmt, st1, st2);
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 	exit(1);
 }
 
